import pylab as pyl
import numpy as np
import scipy as sy
import scipy.fftpack as syfp
from scipy import signal
from scipy.fftpack import fft, ifft
import matplotlib.pyplot as plt
import pandas as pd
import os
from scipy.signal import find_peaks_cwt
import csv
import heapq
import datetime
import pandas as pd
import numpy as np
from datetime import datetime, timedelta
import pandas as pd
import scipy.signal
import numpy as np
import collections
from numpy import (array, dot, arccos, clip)
from numpy.linalg import norm
from configparser import ConfigParser

import timeit
import time
import math
from collections import Counter

class Angle:
	def __init__(self,horizontAccelData, verticalAccelData):
		self.df1 = horizontAccelData
		self.df2 = verticalAccelData



	def angleis(self):
		try:
			v1=self.df1
			v2=self.df2
			c = dot(v1,v2)/norm(v1)/norm(v2)
			angle = arccos(clip(c, -1, 1)) 
			AngleinDegree = (angle*360)/(2*3.14)
			# print(AngleinDegree)
			return AngleinDegree
		except Exception as e:
			print(str(e))
	def CallerMain(self):
		self.angleis()

	# Below is online code, since I have used it inside dataframe, below use self.df1['v'] for horizont and vertical
	def phasefind(self):
		try:
			horizontalaxesamplitude = list(self.df1)#['v'])
			verticalamplitude = list(self.df2)#['v'])
			if len(self.df2)!=len(self.df1):
				difference = abs(len(horizontalaxesamplitude)-len(verticalamplitude))
				if len(horizontalaxesamplitude)>len(verticalamplitude):
					horizontalaxesamplitude = horizontalaxesamplitude[: -difference or None]  #to delete last n elemenets
					verticalamplitude = verticalamplitude
				elif len(verticalamplitude)>len(horizontalaxesamplitude):
					verticalamplitude = verticalamplitude[: -difference or None] 
					horizontalaxesamplitude = horizontalaxesamplitude
				amplitudeX =  horizontalaxesamplitude 
				amplitudeY =  verticalamplitude 
				c = dot(amplitudeX,amplitudeY)/norm(amplitudeX)/norm(amplitudeY)
				angle = arccos(clip(c, -1, 1))
				angleindegree = ((angle*360)/(2*3.14))
				# print('angle in degree',((angle*360)/(2*3.14)))
				return angleindegree
			else:
				amplitudeX =  horizontalaxesamplitude 
				amplitudeY =  verticalamplitude
				c = dot(amplitudeX,amplitudeY)/norm(amplitudeX)/norm(amplitudeY)
				angle = arccos(clip(c, -1, 1))
				angleindegree = ((angle*360)/(2*3.14))
				# print('angle in degree',((angle*360)/(2*3.14)))
				return angleindegree
		except Exception as e:
			print(str(e))

# for_phase_accel_Horizon = pd.read_csv(r'D:\vegam_csv_files\Literature_data\normal_Mafaulda\Downsampled_and_sending_vMaint\acceleromter_values_downsampled\v3_20.2752.csv')
# for_phase_accel_verti = pd.read_csv(r'D:\vegam_csv_files\Literature_data\normal_Mafaulda\Downsampled_and_sending_vMaint\acceleromter_values_downsampled\25.csv')
# v1 = for_phase_accel_Horizon['v']
# v2 = for_phase_accel_verti['v']


# angle = Angle(v1,v2)
# # print(angle.phasefind())



# -*- coding: utf-8 -*-
"""
Created on Fri Dec 10 18:35:09 2021

@author: laptop58
"""


# imported libraries
import numpy as np
import math
import vanalytics_helper as v_helper
# import logging
# import logers as log_file
import itertools
import rpm as shaftspeed
from util2 import ParserFftCalculation as fft
import vmaint_constants as constants
import json
import time
from nltk import flatten


class Gear(object):
    #log = log_file.customLogger(logging.DEBUG)

    def __init__(self, operating_rpm_X, number_of_teeth_pinion, samplingFrequency, nsamp, 
                 windowsize, equipment_id, sensor_id, avgfft_data_X, avgfft_data_Y, avgfft_data_Z,freqn, 
                 freqn_X, freqn_Y, freqn_Z, rms):
        '''
           samplingFrequency : Sampling Frequency of the sensor
           nsamp : Number of samples
           windowsize : window size of the fft data
           max_rpm : Maximum speed of the Motor
           number_of_teeth_pinion: number of teeths of gear
           fftlist : FFT Data
           '''
        self.limit_value = float(0.1)
        #self.operatingRpm = float(operatingRpm)
        self.number_of_teeth_pinion = int(number_of_teeth_pinion)
        self.sampling_frequency = int(samplingFrequency)
        self.no_sample = int(nsamp)
        self.window_Size = int(windowsize)
        self.avgfft_data_X = avgfft_data_X
        self.avgfft_data_Y = avgfft_data_Y
        self.avgfft_data_Z = avgfft_data_Z
        self.freqn=freqn
        self.freqn_X = freqn_X
        self.freqn_Y = freqn_Y
        self.freqn_Z = freqn_Z
        self.operating_rpm_X = operating_rpm_X
        self.equipment_id = equipment_id
        self.sensor_id = sensor_id
        self.limit2 = []
        #self.frequency = frequency
        self.rms = rms
        self.freqn_data = []

    def Start_End_Array(self, fftdata, freqn, operatingRpm,limit):
       # Shaft Rotational Speed
       try:
            if operatingRpm is not None:
                FFT_DF = fftdata
                #limit = operatingRpm * constants.THIRTY_PERCENT
                for j in range(1, 11):
                    Rpm = (operatingRpm)*j
                    lower_limit, upper_limit = Rpm - limit, Rpm + limit
                    harmonic_value = [idx for idx, element in enumerate(
                        freqn) if lower_limit <= element <= upper_limit]
                    
                    amplitude_data = [FFT_DF[i] for i in harmonic_value]
                    frequency_values = [freqn[i] for i in harmonic_value]
                    #max_value_1 = max(amplitude_data)
                    #self.limit2.append(max_value_1)
                    self.freqn_data.append(frequency_values)
                    freqn_list = flatten(self.freqn_data)
                return freqn_list

            else:
                return
       except Exception as e:
           print(e)
    # speed and number of teeths are the inputs to the gear analysis

    def gear_mesh_frequency(self, operatingRpm):
        try:
            GMFF = float(operatingRpm * self.number_of_teeth_pinion)

            return GMFF
        except Exception as e:
           print(e)
        # except:
        #     self.log.error(
        #         "Exception occurred while calculating gear mesh frequency", exc_info=True)

    # fault ranges of gear mesh frequency is calculated below
    def fault_range(self, input_value):
        try:
            if input_value is not None:

                # 67+13.14=76
                finalrange_high = float(input_value+1)
                finalrange_high2 = float(input_value+2)

                # 67-13.4=56
                finalrange_low = float(input_value-1)
                finalrange_low2 = float(input_value-2)

                return finalrange_low, finalrange_high, finalrange_high2, finalrange_low2
            else:
                return
        except Exception as e:
           print(e)
        # except:
        #     self.log.error(
        #         "Exception occurred while calculating gear ranges", exc_info=True)

    # checks whether gear mesh frequency range matches the the frequency greater then rms obtained from the parser
    #   11 - No Gear Fault
    #   12 - Gear Fault
    def frequency_call(self,operatingRpm):
        try:
            limit1=operatingRpm*constants.THIRTY_PERCENT
            limit2=operatingRpm*constants.FORTY_PERCENT
            if self.no_sample==8192 and self.window_Size == 8192:
                Frequencies_x = self.Start_End_Array(
                    self.avgfft_data_X, self.freqn, self.operating_rpm_X,limit1)
                frequencies_y = self.Start_End_Array(
                    self.avgfft_data_Y, self.freqn, self.operating_rpm_X,limit1)
                frequencies_z = self.Start_End_Array(
                    self.avgfft_data_Z, self.freqn, self.operating_rpm_X,limit1)
            else:
                Frequencies_x = self.Start_End_Array(
                    self.avgfft_data_X, self.freqn, self.operating_rpm_X,limit2)
                frequencies_y = self.Start_End_Array(
                    self.avgfft_data_Y, self.freqn, self.operating_rpm_X,limit2)
                frequencies_z = self.Start_End_Array(
                    self.avgfft_data_Z, self.freqn, self.operating_rpm_X,limit2)
            return Frequencies_x
        except Exception as e:
            print(e)
    def gmff(self, fault_freqn_list):
        try:
            frequencies_x = self.frequency_call(self.operating_rpm_X)
            if frequencies_x is not None:
                gear_list = []
                gear_result = "11"
                if any(x in fault_freqn_list for x in frequencies_x):
                    gear_result = "12"
               
                    
                if any(x in 2*fault_freqn_list for x in 2*frequencies_x):
                    gear_result = "12"
              

                if any(x in 3*fault_freqn_list for x in 3*frequencies_x):
                    gear_result = "12"
                gear_list.append(gear_result)
                return gear_list
            else:
                return
        except Exception as e:
           print(e)
        # except Exception:
        #     self.log.critical(
        #         "Exception occurred while comparing the calculated gear harmonic ranges to obtained ranges through fft", exc_info=True)

    # all the above methods are called here
    def gmff_total_check(self):
        try:
            gmff = self.gear_mesh_frequency(self.operating_rpm_X)
            if gmff is not None:
                gmff_values = list(self.fault_range(gmff))
                gear_result = self.gmff(gmff_values)

                data = [self.equipment_id, self.sensor_id, "F",
                        json.dumps(gear_result) + "$T$" + str(time.time())]
                # Stop individual data publish
                # v_helper.helper.data_to_publish.append(data)
                return gear_result

            else:
                return
        except Exception as e:
            print(e)
        # except:
        #     self.log.critical(
        #         "Exception occurred while calling the functions", exc_info=True)



import logging
import sys
import math
#   external library
import numpy as np
import pandas as pd
import vanalytics_helper as v_helper
import logers as log_file
import itertools
import vmaint_constants as constants

# ParserFftCalculation  class calculates and returns the AVERAGE, rms, frequencies whose amplitude are >3*rms


class ParserFftCalculation(object):
    #log = log_file.customLogger(logging.DEBUG)

    @classmethod
    def parse_fft_rawdata(cls, sampling_frequency, fft_data, rms_threshold_range = constants.THREE):
        try:
            fft_average = [sum(e)/len(e) for e in zip(*fft_data)]
            # frequencies are selected
            xf = np.linspace(constants.START_RANGE, constants.END / (constants.TWO *
                                                                     (constants.END / sampling_frequency)), len(fft_average))
            # rms is calculated using average of FFT
            rms = np.sqrt(np.mean(np.square(fft_average)))

            # list of all indices [looseness, misalignment, unbalanace]
            index = [list(fft_average).index(i) for i in fft_average]
            amplitude = [i for i in fft_average]
            #frequency = [list(xf)[i] for i in index]
            frequency=list(xf)
            # checks for amplitudes index greater than 3 times of RMS [bearing, gearbox]
            rms_index = [list(fft_average).index(
                i) for i in fft_average if i > rms_threshold_range * rms]
            rms_amplitude = [list(fft_average).index(
                i) for i in fft_average if i > rms_threshold_range * rms]
            rms_frequency = [list(xf)[i] for i in rms_index]
            

            
            
            # # checks for amplitudes index greater thn 3*rms
            # amplitude_rms_index = [list(fft_average).index(
            #     i) for i in fft_average if i > rms_threshold_range * rms_of_fft_average]
            # # checks for the amplitudes for the index values
            # horizontamplitudabove_rms_values = [
            #     i for i in fft_average if i > rms_threshold_range * rms_of_fft_average]
            # # corresponding freqns of amplitudes greater than 3*rms
            # frequency_above_rms_values = [list(xf)[i]
            #                               for i in amplitude_rms_index]

            output = [fft_average, rms, amplitude, frequency, rms_amplitude, rms_frequency]
            return output

        except Exception as e:
            print(e)


import os
import pandas as pd
import logging
import numpy as np
import threading
import time
import json
import requests as req
from nltk import flatten
import logers
import vanalytics_data as v_data
import vanalytics_api as v_api
import vanalytics_helper as v_helper
import mqtt_con as m_con

log = logers.customLogger(logging.DEBUG)

DIR = os.path.dirname(__file__)
if not DIR:
    FILE_PATH = "main_config.json"
else:
    FILE_PATH = DIR + "/main_config.json"

with open(FILE_PATH, 'r') as readfile:
    main_config = json.load(readfile)
    v_data.CommonFeatures.main_config = main_config

log.info('starting vegam analytics app')


def host_vanalytics():
    try:
        x = threading.Thread(target=v_api.host_api, args=(), daemon=True)
        x.start()
    except Exception as ex:
        log.critical('Exception occured while hosting the api', exc_info=True)
        print(ex)


def publish_data():
    while True:
        if len(v_helper.helper.data_to_publish) > 0:
            data = None
            try:
                data = v_helper.helper.data_to_publish.pop()
                pub_equipment_id = data[0]
                pub_sensor_id = data[1]
                pub_type = data[2]
                # pub_payload = json.dumps(data[3])
                pub_payload = data[3]
                broker = getBrokerDetails(pub_equipment_id)
                topic = getPublishTopics(
                    pub_equipment_id, pub_sensor_id, pub_type)
                if topic != "-1":                    
                    client = broker.MqttClient
                    if(pub_type == "FO"):
                        #topic = topic + "All"
                        topic = topic
                    client.publish_data(topic, pub_payload)
                    print(f"{topic} : {pub_payload}")
            except Exception as ex:
                log.critical(
                    f"ERROR while publishing data {data}", exc_info=True)

        time.sleep(.05)


def getBrokerDetails(equipment_id): 
    try:
        brokerinfo = v_data.CommonFeatures.equipment_store[equipment_id]['E'].Brokers[0]
        output = brokerinfo.IpAddress + "_" + str(brokerinfo.PortNumber)
        broker = v_data.CommonFeatures.brokers_dict[output]
        return broker
    except Exception as ex:
        print(
            f"Error while getting broker details for equipment {equipment_id}, with error {ex}")
        log.critical(
            f"Error while getting broker details for equipment {equipment_id}", exc_info=True)

def getPublishTopics(equipment_id, sensor_id, tagType):
    topic = "-1"
    try:
        if tagType.upper() == "H":
            tagType = "HEALTH"
        elif tagType.upper() == "O":
            tagType = "ONOFF"
        elif tagType.upper() == "R":
            tagType = "RPM"
        else:
            tagType = "FAULT"

        sensors = v_data.CommonFeatures.equipment_store[equipment_id]['E'].AttachedSensors
        sensor = next((x for x in sensors if x.SensorMacID == sensor_id), None)
        tags = sensor.Tags
        tag = next((x for x in tags if x.TopicRepresents == tagType), None)
        topic = tag.Topic
    except Exception as ex:
        print(ex)
        #   write log here
    finally:
        return topic

queue_read_flag = True

host_vanalytics()

# handle startup here
def main_start():
    #   get the list of site id's
    v_data.CommonFeatures.site_ids = v_data.CommonFeatures.get_site_to_moniter()

    #   if site are there, get the list of equipments
    if v_data.CommonFeatures.site_ids is not None and len(v_data.CommonFeatures.site_ids) > 0:
        v_data.CommonFeatures.get_sites_equipments()
    else:
        v_data.CommonFeatures.site_ids = []
        log.warning("vmaint api does not returned any sites to monitor")

    #   if metadata list has items, fetch them
    if len(v_data.CommonFeatures.metadata_lookup) > 0:
        v_data.CommonFeatures.get_metadata()

    #   check if brokers to connect
    broker_keys = list(v_data.CommonFeatures.brokers_dict.keys())
    if len(broker_keys) > 0:
        v_data.CommonFeatures.connect_subscribe_mqtt(broker_keys)

def time_based_data_processing():
    while True:
        try:
            keys = list(v_data.CommonFeatures.sensordata_lookup.keys())
            if len(keys) > 0:
                for key in keys:
                    vsens = v_data.CommonFeatures.sensordata_lookup[key]
                    if vsens.BeingProcessed == False:
                        can_process = vsens.check_conditions_for_algo_process(True)

                        if can_process:
                            stream_process_th = threading.Thread(target=vsens.process_burst, args=(), daemon=True)
                            stream_process_th.start()
        except Exception as ex:
            print(ex)
        finally:
            time.sleep(60)

main_start()

publish_thread = threading.Thread(target=publish_data, args=(), daemon=True)
publish_thread.start()

dp_thread = threading.Thread(target=time_based_data_processing, args=(), daemon=True)
dp_thread.start()

while True:
    if v_helper.helper.data_queue.empty() == False:
        try:
            input_data = v_helper.helper.data_queue.get()
            if len(input_data) == 2:
                topic = input_data[0]
                raw_data = input_data[1]
                if topic == "se_poc/rpm":
                    if raw_data == "0":
                        v_data.CommonFeatures.simulator_rpm = 0
                    else:
                        v_data.CommonFeatures.simulator_rpm = float(raw_data)
                    continue

                topic_data = v_data.CommonFeatures.topics_lookup[topic]
                if topic_data is not None:
                    equipment_id = topic_data['E']
                    sensor_id = topic_data['S']
                    tag_type = topic_data['T']

                    vsens = None
                    if sensor_id in v_data.CommonFeatures.sensordata_lookup:                        
                        vsens = v_data.CommonFeatures.sensordata_lookup[sensor_id]

                    if vsens is None:
                        vsens = v_data.VsensStreamProcessor(sensor_id, equipment_id)
                        print(f"creating object for {sensor_id}")
                        if vsens.update_dataset() is None:
                            continue
                         
                        v_data.CommonFeatures.sensordata_lookup[sensor_id] = vsens

                    if vsens.TempStore is None:
                        vsens.axis_selection()
                    
                    vsens.store_data(tag_type, raw_data)
                    can_process = vsens.check_conditions_for_algo_process()

                    if can_process:
                        stream_process_th = threading.Thread(target=vsens.process_burst, args=(), daemon=True)
                        stream_process_th.start()                    
                else:
                    log.error(
                        f"topic {topic} is not available in equipment store")
            else:
                log.error(
                    f"Invalid data received {input_data}", exc_info=True)
        except Exception as ex:
            log.error(
                f"Error while processing the data dequeue with error: {ex}", exc_info=True)
        #finally:
            #time.sleep(0.005)


# TODO handle the stoping of application
'''
def stop_processing():
    #   stop the hosted flask api
    #   kill all long running threads
    #   release all the resources
    #   kill main thread
    pass
'''


# -*- coding: utf-8 -*-
"""
Created on Fri Dec 10 17:39:56 2021

@author: laptop58
"""

import logging
import sys
import math
# external library
import numpy as np
import pandas as pd
import vanalytics_helper as v_helper
import logers as log_file
import itertools
import util as fft
import vmaint_constants as constants
import time
import json
import os
from nltk import flatten

DIR = os.path.dirname(__file__)
if not DIR:
    FILE_PATH = "bearing_config.json"
else:
    FILE_PATH = DIR + "/bearing_config.json"

bearing_config = None
with open(FILE_PATH, 'r') as readfile:
    bearing_config = json.load(readfile)


class Bearing(object):
    #log = log_file.customLogger(logging.DEBUG)

    def __init__(self, number_of_balls, inner_diameter, outer_diameter, angle, partNumber, operating_rpm_X, 
                 sampling_frequency, number_sample, window_size, equipment_id, sensor_id, avgfft_data_X, 
                 avgfft_data_Y, avgfft_data_Z,freqn, freqn_X, freqn_Y, freqn_Z, rms):
        '''samplingFrequency : Sampling Frequency of the sensor
           nsamp : Number of samples 
           windowsize : window size of the fft data
           operatingRpm : rpm of the Motor
           number_of_balls : number of balls present in bearing
           inner_diameter :inner_diameter of bearing
           outer_diameter :outer_diameter of bearing
           angle : contact angle of bearing
           fftlist : FFT Data
        '''
        self.limit_value = float(0.1)
        self.number_of_balls = 0 if number_of_balls is None else int(
            number_of_balls)
        self.inner_diameter = 0.0 if inner_diameter is None else float(
            inner_diameter)
        self.outer_diameter = 1.0 if outer_diameter is None else float(
            outer_diameter)
        self.angle = 0 if angle is None else float(angle)
        self.sampling_frequency = sampling_frequency
        self.number_sample = number_sample
        self.window_size = window_size
        self.equipment_id = equipment_id
        self.sensor_id = sensor_id
        self.avgfft_data_X = avgfft_data_X
        self.avgfft_data_Y = avgfft_data_Y
        self.avgfft_data_Z = avgfft_data_Z
        self.freqn=freqn
        self.freqn_X = freqn_X
        self.freqn_Y = freqn_Y
        self.freqn_Z = freqn_Z
        self.operating_rpm_X = operating_rpm_X
       
        self.rms_threshold = constants.THREE
        self.partNo = partNumber
        self.freqn_data = []
        self.limit2 = []
        self.hrm = []
        self.rms = rms
    # bearing characterstic frequencies are calculated in the below method
    # number_of_balls,inner_diameter,outer_diameter,angle are bearing inputs

    def bearing_characterstic_frequency(self, operatingRpm):
        try:
            if self.partNo is not None:
                self.partNo = self.partNo.strip().upper().replace(" ", "")
                if len(self.partNo) > 0:
                    return self.get_bearing_freq_using_partNo(operatingRpm)

            return self.get_bearing_freq(operatingRpm)
        except Exception as e:
            print(e)

        # except:
        #     self.log.error(
        #         "Exception occurred while calculating bearing characterstic frequencies using part number", exc_info=True)

    def get_bearing_freq_using_partNo(self, operatingRpm):
        try:
            cage_factor = bearing_config[self.partNo]["cage"]
            ballspin_factor = bearing_config[self.partNo]["ballspin"]
            innerrace_factor = bearing_config[self.partNo]["innerace"]
            outerrace_factor = bearing_config[self.partNo]["outerrace"]

            cage_freq = int(cage_factor * operatingRpm)
            ballspin_freq = int(ballspin_factor * operatingRpm)
            innerrace_freq = int(innerrace_factor * operatingRpm)
            outerrace_freq = int(outerrace_factor * operatingRpm)

            return [innerrace_freq, outerrace_freq, ballspin_freq, cage_freq]
        except Exception as e:
            print(e)
        # except:
        #     self.log.error(
        #         "Exception occurred while calculating bearing characterstic frequencies using part number", exc_info=True)

    def get_bearing_freq(self, operatingRpm):
        try:
            # all four bearing characterstic frequencies are calculated
            cage_therotical = (
                (constants.ONE/constants.TWO)*(constants.ONE-(self.inner_diameter/self.outer_diameter)*math.cos(self.angle)))
            # fundamental train frequency is calculated using formula
            cage = cage_therotical * operatingRpm
            cage_result = int(cage)

            innerrace_therotical = (
                (self.number_of_balls/constants.TWO)*(constants.ONE+(self.inner_diameter/self.outer_diameter)*math.cos(self.angle)))
            # ball pass frequency of inner race is calculated using formula
            innerrace = innerrace_therotical * operatingRpm
            innerace_result = int(innerrace)

            outerrace_therotical = (
                (self.number_of_balls/constants.TWO)*(constants.ONE-(self.inner_diameter/self.outer_diameter)*math.cos(self.angle)))
            # ball pass frequency  of outer race is calculated using formula
            outerrace = outerrace_therotical * operatingRpm
            outerrace_result = int(outerrace)

            ball_spin_therotical = ((self.outer_diameter/(constants.TWO*self.inner_diameter))*(
                constants.ONE-((self.inner_diameter/self.outer_diameter)*(math.cos(self.angle)))**constants.TWO))
            # ball spin frequency is calculated using formula
            ballspin = ball_spin_therotical * operatingRpm
            ballspin_result = int(ballspin)

            return [innerace_result, outerrace_result, ballspin_result, cage_result]
        except Exception as e:
           print(e)
        # except:
        #     self.log.error(
        #         "Exception occurred while calculating bearing characterstic frequencies", exc_info=True)

    def Start_End_Array(self, fftdata, freqn, operatingRpm,limit):
        # Shaft Rotational Speed
        try:
            if operatingRpm is not None:
                FFT_DF = fftdata
                #limit = operatingRpm * constants.THIRTY_PERCENT
                for j in range(1, 11):
                    Rpm = (operatingRpm)*j
                    lower_limit, upper_limit = Rpm - limit, Rpm + limit
                    harmonic_value = [idx for idx, element in enumerate(
                        freqn) if lower_limit <= element <= upper_limit]
                    #self.hrm.append(harmonic_value)
                    amplitude_data = [FFT_DF[i] for i in harmonic_value]
                    frequency_values = [freqn[i] for i in harmonic_value]
                    #max_value_1 = max(amplitude_data)
                    #self.limit2.append(max_value_1)
                    self.freqn_data.append(frequency_values)
                    freqn_list = flatten(self.freqn_data)
                return freqn_list

            else:
                return
        except Exception as e:
            print(e)

    def faultrange(self, input_value):
        try:
            if input_value is not None:

                # 67+13.14=76
                finalrange_high = float(input_value+1)
                finalrange_high2 = float(input_value+2)

                # 67-13.4=56
                finalrange_low = float(input_value-1)
                finalrange_low2 = float(input_value-2)

                return finalrange_low, finalrange_high, finalrange_high2, finalrange_low2
            else:
                return
        except Exception as e:
           print(e)
    def frequency_call(self,operatingRpm):
        try:
            limit1=operatingRpm*constants.THIRTY_PERCENT
            limit2=operatingRpm*constants.FORTY_PERCENT
            if self.number_sample==8192 and self.window_size==8192:
                Frequencies_x = self.Start_End_Array(
                    self.avgfft_data_X, self.freqn, self.operating_rpm_X,limit1)
                frequencies_y = self.Start_End_Array(
                    self.avgfft_data_Y, self.freqn, self.operating_rpm_X,limit1)
                frequencies_z = self.Start_End_Array(
                    self.avgfft_data_Z, self.freqn, self.operating_rpm_X,limit1)
            else:
                Frequencies_x = self.Start_End_Array(
                    self.avgfft_data_X, self.freqn, self.operating_rpm_X,limit2)
                frequencies_y = self.Start_End_Array(
                    self.avgfft_data_Y, self.freqn, self.operating_rpm_X,limit2)
                frequencies_z = self.Start_End_Array(
                    self.avgfft_data_Z, self.freqn, self.operating_rpm_X,limit2)
            return Frequencies_x
        except Exception as e:
            print(e)
    # except:
        #     self.log.error(
        #         "Exception occurred while calculating ranges of bearing characterstic frequency", exc_info=True)
     # checks whether cage therotical frequencies  range matches the the frqns greater then rms obtained from the parser
    def cage(self, fault_freqn_list):
        try:
            frequencies_x = self.frequency_call(self.operating_rpm_X)
                
            if frequencies_x is not None:
                cage_result = []
                if any(x in fault_freqn_list for x in frequencies_x):

                    bearing_result = "4"
                    cage_result.append(bearing_result)
                else:
                    bearing_result = "0"
                    cage_result.append(bearing_result)

                # second harmonic check
                if any(x in 2*fault_freqn_list for x in 2*frequencies_x):

                    bearing_result = "4"
                    cage_result.append(bearing_result)

                else:
                    bearing_result = "0"
                    cage_result.append(bearing_result)

                # third harmonic check
                if any(x in 3*fault_freqn_list for x in 3*frequencies_x):

                    bearing_result = "4"
                    cage_result.append(bearing_result)

                else:
                    bearing_result = "0"
                    cage_result.append(bearing_result)

                return cage_result
            else:
                return
        except Exception as e:
           print(e)
        # except:
        #     self.log.critical(
        #         "Exception occurred while comparing the calculated cage harmonic ranges to obtained ranges through fft", exc_info=True)

    # checks whether ball spin therotical frequencies  range matches the the frqns greater then rms obtained from the parser
    def ballspin(self, fault_freqn_list):
        try:
            frequencies_x = self.frequency_call(self.operating_rpm_X)

            if frequencies_x is not None:

                ballspin_result = []

                if any(x in fault_freqn_list for x in frequencies_x):
                    bearing_result = "3"
                    ballspin_result.append(bearing_result)

                else:
                    bearing_result = "0"
                    ballspin_result.append(bearing_result)
                # second harmonic check
                if any(x in 2*fault_freqn_list for x in 2*frequencies_x):

                    bearing_result = "3"
                    ballspin_result.append(bearing_result)
                else:
                    bearing_result = "0"
                    ballspin_result.append(bearing_result)

                # third harmonic check
                if any(x in 3*fault_freqn_list for x in 3*frequencies_x):

                    bearing_result = "3"
                    ballspin_result.append(bearing_result)
                else:
                    bearing_result = "0"
                    ballspin_result.append(bearing_result)

                return ballspin_result
            else:
                return
        except Exception as e:
           print(e)
        # except:
        #     self.log.critical(
        #         "Exception occurred while comparing the calculated ball spin harmonic ranges to obtained ranges through fft", exc_info=True)

    # checks whether outerrace therotical frequencies  range matches the the frqns greater then rms obtained from the parser
    def outerrace(self, fault_freqn_list):
        try:
            frequencies_x = self.frequency_call(self.operating_rpm_X)
            outerrace_result = []

            if frequencies_x is not None:

                if any(x in fault_freqn_list for x in frequencies_x):

                    bearing_result = "2"
                    outerrace_result.append(bearing_result)
                else:
                    bearing_result = "0"
                    outerrace_result.append(bearing_result)

                # second harmonic check
                if any(x in 2*fault_freqn_list for x in 2*frequencies_x):
                    bearing_result = "2"
                    outerrace_result.append(bearing_result)
                else:
                    bearing_result = "0"
                    outerrace_result.append(bearing_result)

                # third harmonic check
                if any(x in 3*fault_freqn_list for x in 3*frequencies_x):
                    bearing_result = "2"
                    outerrace_result.append(bearing_result)
                else:
                    bearing_result = "0"
                    outerrace_result.append(bearing_result)

                return outerrace_result
            else:
                return
        except Exception as e:
           print(e)
        # except:
        #     self.log.critical(
        #         "Exception occurred while comparing the calculated Outerrace harmonic ranges to obtained ranges through fft", exc_info=True)

    # checks whether innerrace therotical frequencies  range matches the the frqns greater then rms obtained from the parser
    def innerrace(self, fault_freqn_list):
        try:
            frequencies_x = self.frequency_call(self.operating_rpm_X)

            innerrace_result = []
            if frequencies_x is not None:
                if any(x in fault_freqn_list for x in frequencies_x):

                    bearing_result = "1"
                    innerrace_result.append(bearing_result)
                else:
                    bearing_result = "0"
                    innerrace_result.append(bearing_result)

                if any(x in 2*fault_freqn_list for x in 2*frequencies_x):

                    bearing_result = "1"
                    innerrace_result.append(bearing_result)
                else:
                    bearing_result = "0"
                    innerrace_result.append(bearing_result)
                if any(x in 3*fault_freqn_list for x in 3*frequencies_x):

                    bearing_result = "1"
                    innerrace_result.append(bearing_result)
                else:
                    bearing_result = "0"
                    innerrace_result.append(bearing_result)

                return innerrace_result
            else:
                return
        except Exception as e:
          print(e)
        # except:
        #     self.log.critical(
        #         "Exception occurred while comparing the calculated innerrace harmonic ranges to obtained ranges through fft", exc_info=True)

    # all the above methods are called one by one
    def bearing_total_check(self):

        try:
            fault_freq = self.bearing_characterstic_frequency(
                self.operating_rpm_X)
            if fault_freq is not None and len(fault_freq) == 4:
                inner = fault_freq[0]
                outer = fault_freq[1]
                ball = fault_freq[2]
                cages = fault_freq[3]

                faultrange_cage = list(self.faultrange(
                    cages))

                faultrange_ballspin = list(self.faultrange(
                    ball))

                faultrange_outerrace = list(self.faultrange(
                    outer))

                faultrange_innerrace = list(self.faultrange(
                    inner))

                cage_result_total = self.cage(
                    faultrange_cage)

                ballspin_result_total = self.ballspin(
                    faultrange_ballspin)

                outerrace_result_total = self.outerrace(
                    faultrange_outerrace)

                innerrace_result_total = self.innerrace(
                    faultrange_innerrace)

                # combining all the above list into a single list
                combined_list = itertools.chain(
                    innerrace_result_total, outerrace_result_total, ballspin_result_total, cage_result_total)
                bearing_algorithm_list = list(set(combined_list))
                # print(bearing_algorithm_list)

                # checks whether all values of combined list is zero
                if all([v == "0" for v in bearing_algorithm_list]):
                    bearing_result_new = "0"
                    bearing_result_new = list(bearing_result_new)
                else:
                    bearing_result_ignore = "0"
                    # removes 0 from the list
                    bearing_result_new = [
                        i for i in bearing_algorithm_list if i not in bearing_result_ignore]
                    # removes duplicates from the list

                # "F" indicates the fault
                data = [self.equipment_id, self.sensor_id, "F", json.dumps(
                    bearing_result_new) + "$T$" + str(time.time())]
                # Stop individual data publish
                # v_helper.helper.data_to_publish.append(data)
                return bearing_result_new

            else:
                return None

        except Exception as e:
            print(e)
        # except:



#importing required packages
from datetime import datetime
import inspect
import logging
from concurrent_log_handler import ConcurrentRotatingFileHandler as crf
import json
import os

#Configuring the log file name
# Opening JSON file
DIR = os.path.dirname(__file__)
if not DIR:
    FILE_PATH = "main_config.json"
else:
    FILE_PATH = DIR + "/main_config.json"

with open(FILE_PATH, 'r') as readfile:
    main_config = json.load(readfile)

file_path= main_config["Log_Path"]
file_time = datetime.now().strftime("%d-%m-%Y_%H-%M-%S")
my_path = file_path + file_time

#defining the logger function and default log level is DEBUG
def customLogger(logLevel=logging.DEBUG):
    # Gets the name of the class / method from where this method is called
    loggerName = inspect.stack()[1][3]
    logger = logging.getLogger(loggerName)
    # By default, log all messages
    logger.setLevel(logLevel)

    #this line helps to generate multiple log files when the log file limit exceeds 20mb
    my_handler = crf("{0}.log".format(my_path), mode='a', maxBytes=20*1024*1024)
    my_handler.setLevel(logLevel)
    # %(name)s will display the class/function name from where it is logging
    formatter = logging.Formatter('%(asctime)s - %(name)s- %(levelname)s: %(message)s',
                                  datefmt='%m/%d/%Y %I:%M:%S %p')
    my_handler.setFormatter(formatter)
    logger.addHandler(my_handler)
    return logger


import os
import logging
import logers
from types import SimpleNamespace
import json
import requests as req
import vanalytics_helper as v_helper
import mqtt_con as m_con
from enum import Enum
import numpy as np
import time

from util import ParserFftCalculation as fft_cal
import rpm as op_rpm

import gear.gear as algo_gearbox
import bearing.bearing as algo_bearing
import velocity_severity.machine_condition as algo_severity
import misalignment.misalignment as algo_misalignment
import looseness.looseness as algo_looseness
import machine_status.on_off as algo_onoff
import unbalance.unbalance as algo_unbalance

import vanalytics_helper as v_helper


log = logers.customLogger(logging.DEBUG)

DIR = os.path.dirname(__file__)
if not DIR:
    FILE_PATH = "main_config.json"
else:
    FILE_PATH = DIR + "/main_config.json"

with open(FILE_PATH, 'r') as readfile:
    main_config = json.load(readfile)

class VegamAlogrithm(Enum):
    VelocitySeverity = 1
    BearingFault = 2
    MisalignmentFault = 3
    UnbalanceFault = 4
    MechanicalLooseness = 5
    GearboxFault = 6
    MachineOnOff = 7

class CommonFeatures:
    #   will have site id list, used to get the equipment from the api for each site
    site_ids = []   
    #   if not able to get euipment, will store those site id here
    site_id_failed = [] 
    #   store the brokers and there respective client { key :- ipaddress_portnumber }
    brokers_dict = {}   

    #   will store id's for which metadata needs to be fetched { 'E': equipment_id & 'M': sensor_id}
    metadata_lookup = []    
    #   if not able to fetch metadata, id's will be stored here
    metadata_lookup_failed = [] 

    #   will store all the topics { 'topic_id': {'E': equipment_id, 'S': sensor_id, 'T': TagType}}
    topics_lookup = {}  
    #   will store the equipment data { equipment_id: {'E': equipment_data, 'M': equipment_metadata }}
    equipment_store = {}
    #   will store the sensor metadata {sensor_id: {'M': sensor_metadata}}    
    sensor_store = {}   
    #   will store data for processing {sensor_id + "_" + topic: [list(raw_data), power, nwindowsize]}
    vs_sensor_rawdata = {}  
    machine_sensor_rawdata = {}
    bearing_sensor_rawdata = {}
    gear_sensor_rawdata = {}
    misalignment_sensor_rawdata = {}
    looseness_sensor_rawdata = {}
    # stores the rawdata for unbalance algorithm
    unbalance_sensor_rawdata = {}
    main_config = {}

    sensordata_lookup = {}

    updated_equipments = {
        VegamAlogrithm.VelocitySeverity: [],
        VegamAlogrithm.MisalignmentFault: [],
        VegamAlogrithm.UnbalanceFault: [],
        VegamAlogrithm.BearingFault: [],
        VegamAlogrithm.GearboxFault: [],
        VegamAlogrithm.MechanicalLooseness: [],
        VegamAlogrithm.MachineOnOff: []
    }

    @classmethod
    def get_site_to_moniter(cls):
        queryStr = "?ipAddress=" + main_config["initial_config"]["ip_address"]
        queryStr = queryStr + "&portNumber=" + str(main_config["initial_config"]["port_number"])
        
        url = main_config["vmaint_api"]["base_url"]
        url = url + "/GetSitesToProcess" + queryStr

        initial_data = None
        try:
            initial_data = req.post(url)
            if initial_data is not None:
                if initial_data.status_code == 200 and initial_data.text != 'null':
                    return json.loads(initial_data.text)
            return []
        except Exception as ex:
            log.error(f'Unable to get the initial list of sites from vmaint api with error: {ex}', exc_info=True)
    
    @classmethod
    def get_sites_equipments(cls):
        if len(CommonFeatures.site_ids) > 0:
            site_id = CommonFeatures.site_ids.pop()
            CommonFeatures.get_equipment_data(site_id)

    @classmethod
    def get_equipment_data(cls, site_id):
        '''
        @param site_id : site_id for which equipment data is required
        '''
        queryStr = "?ipAddress=" + main_config["initial_config"]["ip_address"]
        queryStr = queryStr + "&portNumber=" + str(main_config["initial_config"]["port_number"])
        queryStr = queryStr + "&siteID=" + str(site_id)

        url = main_config["vmaint_api"]["base_url"]
        url = url + "/GetAnalyticsEquipments" + queryStr

        equipment_data = None
        try:
            equipment_data = req.post(url)
            if equipment_data is not None:
                if equipment_data.status_code == 200 and equipment_data.text != 'null':
                    equipment_list = json.loads(equipment_data.text)
                    CommonFeatures.store_equipment_data(equipment_list)
                else:
                    CommonFeatures.site_id_failed.append(site_id)
            else:
                CommonFeatures.site_id_failed.append(site_id)
        except Exception as ex:
            log.error(f'Unable to get the equiment data from vmaint for the site id:- {site_id} with error {ex}', exc_info=True)
            CommonFeatures.site_id_failed.append(site_id)
        finally:
            CommonFeatures.get_sites_equipments()

    @classmethod
    def store_equipment_data(cls, equipment_list):
        '''
        @param equipment_list : list of equipments to be stored
        '''
        for equipment in equipment_list:
            equipment_obj = json.loads(json.dumps(equipment), object_hook=lambda d: SimpleNamespace(**d))                
            if len(equipment_obj.Brokers) == 0:
                continue
            CommonFeatures.equipment_store[equipment_obj.EquipmentID] = {'E' : equipment_obj, 'M' : None }
            CommonFeatures.metadata_lookup.append({ 'E' : equipment_obj.EquipmentID })
            temp = equipment_obj.Brokers[0]
            broker_key = temp.IpAddress + "_" + str(temp.PortNumber)
            if broker_key not in CommonFeatures.brokers_dict:
                broker = v_helper.BrokerInfo(temp.IpAddress, temp.PortNumber, temp.UserName, temp.Password)
                CommonFeatures.brokers_dict[broker_key] = broker
            if len(equipment_obj.AttachedSensors) > 0:
                for sensor in equipment_obj.AttachedSensors:
                    CommonFeatures.metadata_lookup.append({ 'S' : sensor.SensorMacID })
                    if (sensor.SensorMacID in CommonFeatures.sensor_store.keys()) == False:
                        CommonFeatures.sensor_store[sensor.SensorMacID] = None
                    if len(sensor.Tags) > 0:
                        for tag in sensor.Tags:
                            CommonFeatures.topics_lookup[tag.Topic] = { 'E' : equipment_obj.EquipmentID, 'S': sensor.SensorMacID, 'T': tag.TopicRepresents }
                            if tag.TopicType == "S":
                                CommonFeatures.brokers_dict[broker_key].SubTopics.append(tag.Topic)
                            
                        CommonFeatures.brokers_dict[broker_key].SubTopics = list(set(CommonFeatures.brokers_dict[broker_key].SubTopics))

    @classmethod
    def get_metadata(cls):
        if len(CommonFeatures.metadata_lookup) > 0:
            metadata_item = CommonFeatures.metadata_lookup.pop()
            CommonFeatures.call_metadata_api(metadata_item)

    @classmethod
    def call_metadata_api(cls, metadata_item):
        id = ""
        mtype = "E"
        if 'E' in metadata_item:
            id = metadata_item['E']
        else:
            id = metadata_item['S']
            mtype = "S"
        
        post_data = {
            "UniqueName" : id
            , "UniqueId" : ""
            , "Key": []
            , "Tag": []
        }

        url = main_config["metadata_api_info"]["base_url"]

        try:            
            metadata = req.post(url, json=post_data, headers={"Content-Type":"application/json"})
            if metadata is not None:
                if(metadata.status_code == 200 and metadata.text != 'null'):
                    metadata = json.loads(metadata.text)
                    CommonFeatures.store_metadata(metadata, metadata_item, mtype)
                else:
                    CommonFeatures.metadata_lookup_failed.append(metadata_item)
            else:
                CommonFeatures.metadata_lookup_failed.append(metadata_item)
        except Exception as ex:
            log.error(f'Unable to get the equiment/sensor data from metadata for the unique_name:- {id} with error: {ex}', exc_info=True)
            CommonFeatures.metadata_lookup_failed.append(metadata_item)
        finally:
            CommonFeatures.get_metadata()

    @classmethod
    def store_metadata(cls, metadata, metadata_item, mtype):
        try:
            metadata_obj = json.loads(json.dumps(metadata), object_hook=lambda d: SimpleNamespace(**d))
            if mtype == 'E':
                CommonFeatures.equipment_store[metadata_item['E']]['M'] = metadata_obj
            else:
                CommonFeatures.sensor_store[metadata_item['S']] = { 'M' : metadata_obj }
        except Exception as ex:
            log.error(f'Unable to store data {metadata} with error: {ex}', exc_info=True)

    @classmethod
    def connect_subscribe_mqtt(cls, broker_keys):
        '''
        @param broker_key : list of broker key [key: ipaddress + '_' + port] 
        '''
        if len(broker_keys) > 0:
            broker_key = broker_keys.pop()
            try:
                broker_info = CommonFeatures.brokers_dict[broker_key]
                if broker_info.MqttClient is not None and broker_info.MqttClient.isConnected == 1:
                    pass
                else:
                    broker_info.MqttClient = m_con.MqttClient(broker_info.ipAddress, broker_info.portNumber, broker_info.userName, broker_info.password, broker_info.brokerName)
                    broker_info.MqttClient.connect_broker()

                sub_topics = broker_info.SubTopics[:]
                # broker_info.SubTopics = []
                broker_info.MqttClient.subscribe_topic(sub_topics)                
            except Exception as ex:
                log.error(f'Failed to connect/subscribe {broker_key} with error: {ex}', exc_info=True)
            finally:
                CommonFeatures.connect_subscribe_mqtt(broker_keys)

    @classmethod
    def updateEquipment(cls, equipments):
        CommonFeatures.store_equipment_data(equipments)

        if len(CommonFeatures.metadata_lookup) > 0:
            CommonFeatures.get_metadata()

        broker_keys = list(CommonFeatures.brokers_dict.keys())
        if len(broker_keys) > 0:
            CommonFeatures.connect_subscribe_mqtt(broker_keys)

    @classmethod
    def update_equipment_process(cls, equipments):
        for equipment in equipments:
            equipment_obj = json.loads(json.dumps(equipment), object_hook=lambda d: SimpleNamespace(**d))
            CommonFeatures.updated_equipments[VegamAlogrithm.VelocitySeverity].append(equipment_obj.EquipmentID)
            CommonFeatures.updated_equipments[VegamAlogrithm.MisalignmentFault].append(equipment_obj.EquipmentID)
            CommonFeatures.updated_equipments[VegamAlogrithm.UnbalanceFault].append(equipment_obj.EquipmentID)
            CommonFeatures.updated_equipments[VegamAlogrithm.BearingFault].append(equipment_obj.EquipmentID)
            CommonFeatures.updated_equipments[VegamAlogrithm.GearboxFault].append(equipment_obj.EquipmentID)
            CommonFeatures.updated_equipments[VegamAlogrithm.MechanicalLooseness].append(equipment_obj.EquipmentID)
            CommonFeatures.updated_equipments[VegamAlogrithm.MachineOnOff].append(equipment_obj.EquipmentID)

    @classmethod
    def getSensorMetadata(cls, sensor_id, args):
        '''
        @param sensor_id : sensor id for which metadata needs to be searched
        @args : list of metadata keys e.g.['power', 'modelnumber']
        @return : it will return list, [['256'], ['vsens_234']]
        '''
        output = []
        try:
            if sensor_id in CommonFeatures.sensor_store:
                if CommonFeatures.sensor_store[sensor_id] is not None:
                    if CommonFeatures.sensor_store[sensor_id]['M'] is not None:
                        metadata_prop = CommonFeatures.sensor_store[sensor_id]['M'].property
                        for t in args:
                            data = next(
                                (x for x in metadata_prop if x.key.lower() == t.lower()), None)
                            if data is not None:
                                output.append(data.value)
                            else:
                                output.append(None)
        except Exception as ex:
            log.error(
                f"failed to get sensor metadata for the list {args}", exc_info=True)
        finally:
            # TODO need to remove after all data is validated
            return output

    @classmethod
    def getEquipmentData(cls, equipment_id, sensor_id, args):
        '''
        @param equipment_id : equipment id for which data requied
        @param sensor_id : sensor id for which data needs to be searched
        @args : current version supports following keys ['axisx', 'axisy', 'axisz', 'connectionmode']
        @return : it will return list, ['x', 'y', 'z', 'B']
        '''
        output = []
        try:
            if equipment_id in CommonFeatures.equipment_store:
                equipment = CommonFeatures.equipment_store[equipment_id]['E']
                sensors = equipment.AttachedSensors
                req_sensor = next(
                    (x for x in sensors if x.SensorMacID == sensor_id), None)

                if req_sensor is not None:
                    for y in args:
                        y = y.lower()
                        if y == 'axisx':
                            output.append(req_sensor.AxisX)
                        elif y == "axisy":
                            output.append(req_sensor.AxisY)
                        elif y == "axisz":
                            output.append(req_sensor.AxisZ)
                        elif y == "connectionmode":
                            output.append(req_sensor.ConnectionMode)
                        else:
                            output.append(None)
        except Exception as ex:
            print(ex)
            #   write log here
        finally:
            # TODO need to remove after all data is validated
            return output

    @classmethod
    def getEquipmentMetaData(cls, equipment_id, args):
        '''
        @param equipment_id : equipment id for which metada requied
        @param sensor_id : sensor id for which metadata needs to be searched
        @args : list of metadata keys e.g.['power', 'modelnumber']
        @return : it will return list, [['256'], ['vsens_234']]
        '''
        output = []
        try:
            if equipment_id in CommonFeatures.equipment_store:
                data = CommonFeatures.equipment_store[equipment_id]['M']
                if data is not None:
                    metadata_prop = data.property
                    for t in args:
                        data = next(
                            (x for x in metadata_prop if x.key.lower() == t.lower()), None)
                        if data is not None:
                            output.append(data.value)
                        else:
                            output.append(None)
        except Exception as ex:
            log.error(
                f"failed to get equipment metadata for the list {args}", exc_info=True)
        finally:
            # TODO need to remove after all data is validated
            return output


    @classmethod
    def parse_rawdata(cls, data):
        try:
            # type_info = type(data)
            if isinstance(data, dict):
                d = str(data["v"]) + "$T$" + str(data["t"])
            else:
                d = json.loads(data)
                d = str(d["v"]) + "$T$" + str(d["t"])
            return d
        except Exception as ex:
            log.error(f"ERROR: failed to parse {data}")
            print(f"ERROR: failed to parse {data}")

    @classmethod
    def parse_fft_rawdata(cls, data):
        try:
            if isinstance(data, dict):
                d = str(data["v"])
            else:
                d = json.loads(data)
                d = d["v"]
            return d
        except Exception as ex:
            log.error(f"ERROR: failed to parse {data}")
            print(f"ERROR: failed to parse {data}")
       
    @classmethod    
    def process_fft_data(cls, data):
        if not isinstance(data, list) or len(data) < 1:
            return None
        
        try:
            item = json.loads(data[0])
            start_time = item['stime']

            item = json.loads(data[-1])
            end_time = item['etime']

            fft_list = []
            for x in data:
                val = json.loads(x)
                val = val['f']
                fft_list.append(val)
            
            return start_time, end_time, fft_list
        except Exception as ex:
            print(ex)


    @classmethod
    def clear_sensordata_lookup(cls, sensor_id):
        if sensor_id in CommonFeatures.sensordata_lookup:
            del CommonFeatures.sensordata_lookup[sensor_id]

class Axis:
    def __init__(self):
        self.x = []
        self.y = []
        self.z = []

class VibrationParam:
    def __init__(self):
        self.Fft = Axis()
        self.Rms = Axis()
        self.Velocity = Axis()
        self.Displacement = Axis()
        self.Acc = Axis()

class VsensStreamProcessor:
    def __init__(self, sensorID, equipmentID):
        self.SensorID = sensorID
        self.EquipmentID = equipmentID
        self.StreamData = VibrationParam()
        self.WindowSize = None
        self.NoOfSamples = None
        self.SamplingFrequency = None
        self.Window = 0

        self.TempStore = {}
        self.BeingProcessed = False

    def update_dataset(self):
        try:
            output = CommonFeatures.getSensorMetadata(
            self.SensorID, ['numberofsamples', 'windowsize', 'samplingfrequency'])

            if output is not None and len(output) == 3:
                self.NoOfSamples = output[0]
                self.WindowSize = output[1]
                self.SamplingFrequency = output[2]

            if self.NoOfSamples is not None:
                self.NoOfSamples = int(self.NoOfSamples[0])
            else:
                self.NoOfSamples = int(
                    CommonFeatures.main_config["default_meta_values"]["numberofsamples"])

            if self.WindowSize is not None:
                self.WindowSize = int(self.WindowSize[0])
            else:
                self.WindowSize = int(
                    CommonFeatures.main_config["default_meta_values"]["windowsize"])
            
            if self.SamplingFrequency is not None:
                self.SamplingFrequency = int(self.SamplingFrequency[0])
            else:
                self.SamplingFrequency = int(
                    CommonFeatures.main_config["default_meta_values"]["samplingfrequency"])

            # test data
            #self.NoOfSamples = 8192
            #self.WindowSize = 1024
            #self.SamplingFrequency = 6400

            self.Window = int(self.NoOfSamples/self.WindowSize)
            return 0
        except Exception as ex:
            log.error(
                f"Error while updating dataset information for Sensor: {self.SensorID}", exc_info=True)

    def axis_selection(self):
        pass

    def store_data(self, tag_type, raw_data):
        if tag_type.startswith("VELOCITY"):
            raw_data = CommonFeatures.parse_rawdata(raw_data)
            if raw_data is None:
                return

            if tag_type == "VELOCITYX":
                self.StreamData.Velocity.x.append(raw_data)
            elif tag_type == "VELOCITYY":
                self.StreamData.Velocity.y.append(raw_data)
            elif tag_type == "VELOCITYZ":
                self.StreamData.Velocity.z.append(raw_data)

        elif tag_type.startswith("FFT"):
            raw_data = CommonFeatures.parse_fft_rawdata(raw_data)
            if raw_data is None:
                return

            if tag_type == "FFTX":
                self.StreamData.Fft.x.append(raw_data)
            elif tag_type == "FFTY":
                self.StreamData.Fft.y.append(raw_data)
            elif tag_type == "FFTZ":
                self.StreamData.Fft.z.append(raw_data)
        else:
            return

        """elif tag_type.startswith("ACC"):
            raw_data = CommonFeatures.parse_rawdata(raw_data)
            if raw_data is None:
                return

            if tag_type == "ACCX":
                self.StreamData.Acc.x.append(raw_data)
            elif tag_type == "ACCY":
                self.StreamData.Acc.y.append(raw_data)
            elif tag_type == "ACCZ":
                self.StreamData.Acc.z.append(raw_data)"""
    
        
    def check_conditions_for_algo_process(self, is_time_based=False):
        if is_time_based:
           
            if (len(self.StreamData.Fft.x) >= self.Window and len(self.StreamData.Fft.y) >= self.Window
                    and len(self.StreamData.Fft.z) >= self.Window and len(self.StreamData.Velocity.x) >= self.Window
                    and len(self.StreamData.Velocity.y) >= self.Window and len(self.StreamData.Velocity.z) >= self.Window):
                return True
            else:
                return False

    def update_temp_data(self, items):
        if items is None:
            items = ["powerOfMotor", "innerDiameterDriveEnd", "noOfTeethOnPinion", "outerDiameterDriveEnd", "maximumRPM","bearingPartDriveEnd","numberOfBallOrRollerDriveEnd","contactAngleDriveEnd"]
        
        output = CommonFeatures.getEquipmentMetaData(self.EquipmentID, items)
        
        self.TempStore["power"] = None
        self.TempStore["shaftPosition"] = None
        self.TempStore["pinionTeeth"] = None
        self.TempStore["bearingPartNo"] = None
        self.TempStore["maxRpm"] = None
        self.TempStore["numberOfBalls"] = None
        self.TempStore["angle"] = None
        #print(output[0][0] + " " + output[2][0] + " "+ output[4][0])

        #test purpose
        #self.TempStore["power"] = 800
        #self.TempStore["shaftPosition"] = "horizontal"
        #self.TempStore["pinionTeeth"] = 32
        #self.TempStore["bearingPartNo"] = "22410 CC/W23"
        #self.TempStore["maxRpm"] = 997

        try:
            if len(items) == len(output):            
                if output[0][0] is not None:
                    t = output[0][0].strip()
                    if len(t) > 0:
                        self.TempStore["power"] = float(t)
                
                if output[1][0] is not None:
                    t = output[1][0].strip()
                    if len(t) > 0:
                        self.TempStore["innerDiameter"] = t            
                
                if output[2][0] is not None:
                    t = output[2][0].strip()
                    if len(t) > 0:
                        self.TempStore["pinionTeeth"] = int(t)
                if output[3][0] is not None:
                    t = output[1][0].strip()
                    if len(t) > 0:
                        self.TempStore["outerDiameter"] = t   
                            
                if output[5][0] is not None:
                    t = output[5][0].strip()
                    if len(t) > 0:
                        self.TempStore["bearingPartNo"] = t
                
                if output[6][0] is not None:
                    t = output[6][0].strip()
                    if len(t) > 0:
                        self.TempStore["numberOfBalls"] = int(t)
                if output[7][0] is not None:
                    t = output[7][0].strip()
                    if len(t) > 0:
                        self.TempStore["angle"] = float(t)
                if output[4][0] is not None:
                    t = output[4][0].strip()
                    if len(t) > 0:
                        self.TempStore["maxRpm"] = int(t)
        except Exception as ex:
            print(ex)

        
    def process_burst(self):
        if self.BeingProcessed == True:
            return
        
        self.BeingProcessed = True
        try:
            self.update_temp_data(None)

            
            #   parsing velocity
            parsed_velocityX = []
            parsed_velocityY  = []        
            parsed_velocityZ = []        
            for x in self.StreamData.Velocity.x:
                val = x.split("$T$")[0]
                parsed_velocityX.append(float(val))
            avg_velocityX = np.mean(parsed_velocityX)

            for x in self.StreamData.Velocity.y:
                val = x.split("$T$")[0]
                parsed_velocityY.append(float(val))
            avg_velocityY = np.mean(parsed_velocityY)

            for x in self.StreamData.Velocity.z:
                val = x.split("$T$")[0]
                parsed_velocityZ.append(float(val))
            avg_velocityZ = np.mean(parsed_velocityZ)

            #parsing accelerometer data
            """parsed_accX = []
            parsed_accY = []
            parsed_accZ = []
            for x in self.StreamData.Acc.x:
                val = x.split("$T$")[0]
                if len(val) > 0:
                    parsed_accX.append(val)
            for x in self.StreamData.Acc.y:
                val = x.split("$T$")[0]
                if len(val) > 0:
                    parsed_accY.append(val)
            for x in self.StreamData.Acc.z:
                val = x.split("$T$")[0]
                if len(val) > 0:
                    parsed_accZ.append(val)"""

            #   parsing fft
            parsed_fftX = CommonFeatures.process_fft_data(self.StreamData.Fft.x)
            parsed_fftY = CommonFeatures.process_fft_data(self.StreamData.Fft.y)
            parsed_fftZ = CommonFeatures.process_fft_data(self.StreamData.Fft.z)

            # output [fft_average 0, rms 1, amplitude 2, frequency 3, rms_amplitude 4, rms_frequency 5]
            avg_fftX = fft_cal.parse_fft_rawdata(self.SamplingFrequency, parsed_fftX[2])
            avg_fftY = fft_cal.parse_fft_rawdata(self.SamplingFrequency, parsed_fftY[2])
            avg_fftZ = fft_cal.parse_fft_rawdata(self.SamplingFrequency, parsed_fftZ[2])
            fft_bearingX = avg_fftX[5]
            fft_bearingY = avg_fftY[5]
            fft_bearingZ = avg_fftZ[5]
            frequency_X = avg_fftX[3]
            frequency_Y = avg_fftY[3]
            frequency_Z = avg_fftZ[3]

            rms_X = avg_fftX[1]
            rms_Y = avg_fftY[1]
            rms_Z = avg_fftZ[1]
            horizontal_amp_X = avg_fftX[2]
            horizontal_amp_Y = avg_fftY[2]
            horizontal_amp_Z = avg_fftZ[2]
            fft_X = avg_fftX[0]
            fft_Y = avg_fftY[0]
            fft_Z = avg_fftZ[0]
            #   get the RPM
            rpm_from_vib_X = op_rpm.RpmFromVibration(self.SamplingFrequency, self.WindowSize, self.TempStore["maxRpm"],
                                                   self.SensorID, self.EquipmentID, fft_X)
            operating_rpm_X = rpm_from_vib_X.Speed_Detection()
            del rpm_from_vib_X
            
            # velocity severity
            severity = algo_severity.velocity_severity(self.TempStore["power"], avg_velocityX, self.EquipmentID, self.SensorID)
            severity_output = severity.velocity_total_check()

            # machine on off
            onOff = algo_onoff.MachineStatus(fft_X, self.SamplingFrequency, self.EquipmentID, self.SensorID)
            onOff_output = onOff.check_machine_status()
            del onOff

            # bearing
            bearing_output = []
            try:
                bearing_obj = algo_bearing.Bearing(
                    self.TempStore["numberOfBalls"], self.TempStore["innerDiameter"], self.TempStore["outerDiameter"], 
                    self.TempStore["angle"], self.TempStore["bearingPartNo"], operating_rpm_X, self.SamplingFrequency, 
                    self.NoOfSamples, self.WindowSize, self.EquipmentID, self.SensorID, fft_X, fft_Y, fft_Z,frequency_X,avg_fftX[5], avg_fftY[5], avg_fftZ[5], rms_X)
                bearing_output = bearing_obj.bearing_total_check()
                if bearing_output is None:
                    bearing_output = []
            except Exception as ex:
                print(ex)
            # finally:
                # del bearing_obj

            # gearbox
            gear_output = []
            try:
                gear_obj = algo_gearbox.Gear(operating_rpm_X,self.TempStore["pinionTeeth"], self.SamplingFrequency,
                                             self.NoOfSamples, self.WindowSize, self.EquipmentID, self.SensorID, fft_X, fft_Y, fft_Z, frequency_X,avg_fftX[5], avg_fftY[5], avg_fftZ[5], rms_X)
                gear_output = gear_obj.gmff_total_check()
                if gear_output is None:
                    gear_output = []
            except Exception as ex:
                print(ex)
            # finally:
                # del gear_obj

            # misalignment
            misalignment_output = []
            try:
                misalignment_obj = algo_misalignment.Misalignment_Analysis(
                    self.SamplingFrequency, self.NoOfSamples, self.WindowSize, operating_rpm_X, self.SensorID,
                    self.EquipmentID, fft_X, fft_Y, fft_Z, frequency_X, frequency_Y, frequency_Z)
                misalignment_output = misalignment_obj.mis_total_axis_check()
                if misalignment_output is None:
                    misalignment_output = []
            except Exception as ex:
                print(ex)
            # finally:
                # del misalignment_obj

            # looseness
            looseness_output = []
            try:
                looseness_obj = algo_looseness. Looseness_Analysis(
                    self.SamplingFrequency, self.NoOfSamples, self.WindowSize, operating_rpm_X, self.SensorID,
                    self.EquipmentID, fft_X, fft_Y, fft_Z, frequency_X, frequency_Y, frequency_Z)
                looseness_output = looseness_obj.Looseness_Total_Check()
                if looseness_output is None:
                    looseness_output = []
            except Exception as ex:
                print(ex)
            # finally:
                # del looseness_obj

            # unbalance
            unbalance_output = []
            try:
                unbalance_obj = algo_unbalance.Unbalance_Analysis(
                    self.SamplingFrequency, self.NoOfSamples, self.WindowSize, operating_rpm_X, self.SensorID,
                    self.EquipmentID,fft_X, fft_Y, fft_Z,rms_X)
                
                unbalance_output = unbalance_obj.unbalance_total_check()
                if unbalance_output is None:
                    unbalance_output = []
            except Exception as ex:
                print(ex)
            # finally:
                # del unbalance_obj

            final_output = bearing_output + gear_output + misalignment_output + looseness_output + unbalance_output
            data = [self.EquipmentID, self.SensorID, "FO", json.dumps(final_output) + "$T$" + str(time.time())]
            v_helper.helper.data_to_publish.append(data)        
        except Exception as ex:
            print(ex)
        finally:
            self.dispose()

    def dispose(self):
        CommonFeatures.clear_sensordata_lookup(self.SensorID)

    def __del__(self):
        print('Object Destroyed...')



import requests as req
from waitress import serve
import logging
import logers
import os
import json
from flask import Flask, jsonify, request
from flask_api import status
import vanalytics_data as v_data

# log.basicConfig(filename='vanalytics_log.log', filemode='w', format='%(name)s - %(levelname)s - %(asctime)s - %(message)s')
log = logers.customLogger(logging.DEBUG)

DIR = os.path.dirname(__file__)
if not DIR:
    FILE_PATH = "main_config.json"
else:
    FILE_PATH = DIR + "/main_config.json"


with open(FILE_PATH, 'r') as readfile:
    main_config = json.load(readfile)

app = Flask(__name__)

#   hosted api's START
@app.route("/ping", methods=["GET"])
def ping():
    content = {"status": "service is alive"}
    return content, status.HTTP_200_OK

@app.route("/addupdateequipment", methods=["POST"])
def addupdateequipment():
    equipment_list = []
    try:
        equipment_list = request.get_json(force=True)
        if len(equipment_list) == 0:
            content = {"status": "invalid data received"}
            return content, status.HTTP_400_BAD_REQUEST
    except Exception as ex:
        log.error(f"Bad input received {equipment_list}", exc_info=True)
        content = {"status": "invalid data received"}
        return content, status.HTTP_400_BAD_REQUEST
    
    try:
        v_data.CommonFeatures.updateEquipment(equipment_list)
        #v_data.CommonFeatures.update_equipment_process(equipment_list)
        content = {"status": "ok"}
        return content, status.HTTP_200_OK
    except Exception as ex2:
        log.error(f"Failed to add following equipments: {equipment_list}", exc_info=True)
        content = {"status": "internal server error"}
        return content, status.HTTP_500_INTERNAL_SERVER_ERROR
#   hosted api's ENDS

def host_api():
    try:
        ip_address = main_config["initial_config"]["ip_address"]
        port_no = main_config["initial_config"]["port_number"]
        serve(app, host=ip_address, port=port_no)
        log.info(f"Successfully hosted the api at given ip {ip_address} and port {port_no}")
    except Exception as ex:
        log.error(f"Failed to host the api at given ip {ip_address} and port {port_no} with error {ex}", exc_info=True)
        print(ex)
    finally:
        pass


__name__ = "vanalytics_helper"

#import queue
import json

from multiprocessing import Queue

class helper:
    data_to_publish = []
    # data_queue = queue.Queue(maxsize=0)
    data_queue = Queue(maxsize=0)


class BrokerInfo:

    def __init__(self, ipAddress, portNumber, userName, password, brokerName=None, qos=2):
        self.ipAddress = ipAddress
        self.portNumber = portNumber
        self.userName = userName
        self.password = password
        self.brokerName = brokerName
        self.isConnected = 0
        self.qos = qos
        self.MqttClient = None
        self.SubTopics = []

class EquipmentInfo:
    
    def __init__(self, equipmentNumber, equipmentName, equipmentClass, attachedSensors):
        self.equipmentNumber = equipmentNumber
        self.equipmentName = equipmentName
        self.equipmentClass = equipmentClass
        self.SensorInfo = attachedSensors

class SensorInfo:
    
    def __init__(self, sensorIdentifier, connectionMode, axisX, axisY, axisZ, sensorIdentifierOld, tags):
        self.sensorIdentifier = sensorIdentifier
        self.connectionMode = connectionMode
        self.axisX = axisX
        self.axisY = axisY
        self.axisZ = axisZ
        self.sensorIdentifierOld = sensorIdentifierOld
        self.Tags = tags

class Tags:
    
    def __init__(self, topicRepresents, topic, topicType, broker):
        self.topicRepresents = topicRepresents
        self.topic = topic
        self.topicType = topicType
        self.unique_id = None
        self.BrokerInfo = broker

class Entity:

    def __init__(self, unique_id, unique_name, group, description, properties):
        self.unique_id = unique_id
        self.unique_name = unique_name
        self.group = group
        self.description = description
        self.property = properties

class Property:

    def __init__(self, key, value, data_type, tag, range, unit, description):
        self.key = key
        self.value = value
        self.data_type = data_type
        self.tag = tag
        self.range = range
        self.unit = unit
        self.description = description



import numpy as np
import pandas as pd
import os
import time
import vanalytics_helper as v_helper
import logers as l1
import logging
import json
import time

DIR = os.path.dirname(__file__)


if not DIR:
    FILE_PATH = "velocity_severity.csv"
    #FILE_PATH1 = "vs_equipmets.csv"
    FILE_PATH1 = "equipment_severity.json"
else:
    FILE_PATH = DIR + "/velocity_severity.csv"
    #FILE_PATH1 = DIR +"/vs_equipmets.csv"
    FILE_PATH1 = DIR +"/equipment_severity.json"


class velocity_severity(object):
    v_iso_table = pd.read_csv(FILE_PATH)
    #v_input_table = pd.read_csv(FILE_PATH1)
    with open(FILE_PATH1, 'r') as readfile:
        v_input_table = json.load(readfile)


    log = l1.customLogger(logging.DEBUG)

    def __init__(self, power, vel_value, equipment_id, sensor_id):
        self.power = power
        self.vel_value = vel_value
        self.sensor_id = sensor_id
        self.equipment_id = equipment_id
       

    ####checking the class of machine  ####
    ####fun to check the class of machine ###
    def mclass(self):  # power in KW
        try:
            self.power = int(self.power)
            
            if self.power < 15:
                self.classofmotor = 'class1'
            elif self.power in range(15, 75):  # range values are in kW
                self.classofmotor = 'class2'
            elif self.power in range(75, 10000):
                self.classofmotor = 'class3'
            elif self.power > 10000:
                self.classofmotor = 'class4'
            
            return self.classofmotor
        except:
            self.log.error(
                "Exception occurred while checking Class of motor", exc_info=True)
            
        
    #####checking velocity severity level

    def velocity_check(self, motorclass):
        try:
            # velocity iso standard values
            self.viso_val = np.array(
                velocity_severity.v_iso_table['Vrms_mm/s'])
            # difference of velocity values
            self.diff_val = np.abs(self.viso_val-self.vel_value)
            # index of velocity iso values
            self.iso_index = list(self.diff_val).index(min(self.diff_val))
            self.severityLevel = velocity_severity.v_iso_table[motorclass][self.iso_index]

            # returns the level
            return self.severityLevel
        except:
            self.log.error(
                "Exception occurred while checking velocity Severity", exc_info=True)
         

    def velocity_total_check(self):
        try:
            #v_inputfile = velocity_severity.v_input_table
            #vinp_val = v_inputfile['equip_id'].str
            if self.equipment_id in velocity_severity.v_input_table['equipment_ids']:
                #eqp_thrhld_val = v_inputfile.loc[v_inputfile['equip_id'] == self.equipment_id]
                vinp_threshold_low = velocity_severity.v_input_table['equipment_ids'][self.equipment_id]['low']
                vinp_threshold_high = velocity_severity.v_input_table['equipment_ids'][self.equipment_id]['high']

                #vinp_threshold_low = list(eqp_thrhld_val.iloc[:, 3])
                #vinp_threshold_high =list(eqp_thrhld_val.iloc[:, 4])
                #print(vinp_threshold_low,vinp_threshold_high)
                if self.vel_value <= vinp_threshold_low:
                    velocity_check = 0
                    
                elif self.vel_value <= vinp_threshold_high:
                    velocity_check = 1
                    
                else:
                     velocity_check = 2
                    
            else:
                class_of_machine = self.mclass()
                velocity_check = self.velocity_check(class_of_machine)
                if velocity_check.lower() == "good" or velocity_check.lower() == "satisfactory":
                    velocity_check = 0
                elif velocity_check.lower() == "unsatisfactory":
                    velocity_check = 1
                else:
                    velocity_check = 2

            # H - represents "Health" here
            data = [self.equipment_id, self.sensor_id, "H", json.dumps(velocity_check) + "$T$" + str(time.time())]            
            v_helper.helper.data_to_publish.append(data)
            return velocity_check
            
        except:
           self.log.error("Exception occurred while checking velocity Severity Status", exc_info=True)



import sys
import math

#   external library
import numpy as np
import pandas as pd
import vanalytics_helper as v_helper
import itertools
import vmaint_constants as constants

# ParserFftCalculation  class calculates and returns the AVERAGE, rms, frequencies whose amplitude are >3*rms


class ParserFftCalculation(object):
    #log = log_file.customLogger(logging.DEBUG)

    @classmethod
    def parse_fft_rawdata(cls, sampling_frequency, fft_data, rms_threshold_range = constants.THREE):
        try:

            # average of FFT data is done
            fft_average = [sum(e)/len(e) for e in zip(*fft_data)]
            
            # frequencies are selected
            xf = np.linspace(constants.START_RANGE, constants.END / (constants.TWO *
                                                                     (constants.END / sampling_frequency)), len(fft_average))
            # rms is calculated using average of FFT
            rms_of_fft_average = np.sqrt(np.mean(np.square(fft_average)))
            
            # checks for amplitudes index greater than 3 * rms
            amplitude_rms_index = [list(fft_average).index(i) for i in fft_average]
            # checks for the amplitudes for the index values
            horizontamplitudabove_rms_values = [i for i in fft_average]
            # corresponding freqns of amplitudes greater than 3*rms
            frequency_above_rms_values = [list(xf)[i]
                                          for i in amplitude_rms_index]

            output = [fft_average, rms_of_fft_average, horizontamplitudabove_rms_values, frequency_above_rms_values]
            return output

        except Exception as e:
            print(e)
            #self.log.warning("fft data is not in the proper format", exc_info=True)


import pandas as pd
import io


stock_df = pd.read_csv('D:\\Personal\\Jiiva\\DataSet\\tech-stocks-04-2021.csv')

print(stock_df)


class Axis:
    def __init__(self):
        self.x = []
        self.y = []
        self.z = []

class VibrationParam:
    def __init__(self):
        self.Fft = Axis()
        self.Rms = Axis()
        self.Velocity = Axis()
        self.Displacement = Axis()
        self.Acc = Axis()

class VsensProcess:
    def __init__(self):
        self.SensorMacID = ""
        self.VibrationData = VibrationParam()
        self.FirstDataTimestamp = None
        self.FlushoutPeriod = None
        self.Processed = 0
        self.AdditionalInfo = {}
        self.AxisList = []

    def axis_selection(self):
        pass

    def accumulate_data(self):
        pass

    def process_algorithms(self):
        pass

    def dispose(self):
        del self

    def __del__(self):
        print('object destroyed')



mydata = VsensProcess()

print(mydata)
del mydata
print('hello')
#print(mydata)



import numpy as np
import pandas as pd
# import enum
import vanalytics_helper as v_helper
import logers as log
import logging
import util2 as fft_cal
import json
import time
import vmaint_constants as constants


class Misalignment_Analysis(object):

    log = log.customLogger(logging.DEBUG)

    '''
    This is a class to analyse looseness
    '''

    def __init__(self, samplingFrequency, noOfSample, windowsize, operating_rpm, operating_rpm1,
                 sensor_id, equipment_id, avgfft_data_radial, avgfft_data_axial, freqn_axial, freqn_radial):
        '''
        samplingFrequency, windowsize : from sensor details (metadata)
        noOfSample : Number of sampales
        fftlist : FFT data from mqtt
        FREQUENCY_LIMIT = float(0.20)
        RMS_2X_THRESHOLD = float(0.40)
        RMS_3X_THRESHOLD = float(0.30)
        '''
        self.sampling_frequency = samplingFrequency
        self.number_sample = noOfSample
        self.window_size = windowsize
        self.operating_rpm = operating_rpm
        self.operating_rpm1 = operating_rpm1
        self.Equipment_id = equipment_id
        self.Sensor_id = sensor_id
        self.avgfft_data_radial = avgfft_data_radial
        self.avgfft_data_axial = avgfft_data_axial
        self.sub_limits = []
        self.freqn_axial = freqn_axial
        self.freqn_radial = freqn_radial
        self.limit = []
    def Start_End_Array(self):
        # Shaft Rotational Speed
        try:
            if self.operating_rpm and self.operating_rpm1 is not None:
                rpm_ = [self.operating_rpm, self.operating_rpm1]
                for k in range(len(rpm_)):
                    for i in range(1, 7):
                        Rpm = (int(rpm_[k]))*i
                        limits = Rpm-(0.2*Rpm), Rpm+(0.2*Rpm)
                        self.sub_limits.append(limits)
                axial = self.sub_limits[:6]
                axial_value = pd.DataFrame(
                    axial, columns=['start_limit', 'end_limit'])
                #axial_harmonics = axial_value.iloc[[0,1,2,3,4,5]]
                axial_low_limit = axial_value["start_limit"]
                axial_low_limit_toframe = axial_low_limit.to_frame(
                    name="start_limit")
                axial_high_limit = axial_value["end_limit"]
                axial_high_limit_toframe = axial_high_limit.to_frame(
                    name="end_limit")

                radial = self.sub_limits[6:]
                radial_value = pd.DataFrame(
                    radial, columns=['start_limit', 'end_limit'])
                radial_low_limit = radial_value["start_limit"]
                radial_low_limit_toframe = radial_low_limit.to_frame(
                    name="start_limit")
                radial__high_limit = radial_value["end_limit"]
                radial__high_limit_toframe = radial__high_limit.to_frame(
                    name="end_limit")
                return axial_low_limit_toframe, axial_high_limit_toframe, radial_low_limit_toframe, radial__high_limit_toframe
            else:
                return
        except Exception as e:
            print(e)

    def max_val_blueprint(self, data1, data2, fft_data, freqn):
        try:
            for j in range(6):
                amp_low = int(data1.iloc[j, 0])
                amp_high = int(data2.iloc[j, 0])
                data = [idx for idx, element in enumerate(
                    freqn) if amp_low <= element <= amp_high]
                amplitude_data = [fft_data[i] for i in data]
                max_value_1 = max(amplitude_data)
                self.limit.append(max_value_1)
            return self.limit
        except Exception as e:
            print(e)

    def max_val(self):
        try:
            axial_data_low, axial_data_high, radial_data_low, radial_data_high = self.Start_End_Array()
            limit_2_axial = self.max_val_blueprint(
                axial_data_low, axial_data_high, self.avgfft_data_axial, self.freqn_axial)
            limit_2_radial = self.max_val_blueprint(
                radial_data_low, radial_data_high, self.avgfft_data_radial, self.freqn_radial)
            return limit_2_axial, limit_2_radial
        except Exception as e:
            print(e)

    def Misalignment_Check(self, max_amplitude):

        try:
            max_amp1x = max_amplitude[0]
            max_amp_2x = max_amplitude[1]
            max_amp_range1 = constants.ONE_RANGE*max_amp1x
            max_amp_range3 = constants.THREE_RANGE*max_amp1x
            max_amp2 = constants.TWO_RANGE*max_amp1x

            if max_amp_range1 >= max_amp_2x <= max_amp2:
                result = str('acceptable misalignment')

            elif max_amp2 > max_amp_2x <= max_amp_range3:
                result = str('misalignment')
                if max_amp_2x > max_amp_range3:
                   result = str('severe misalignment')

            else:
                result = str('No misalignment')

            return result
        except Exception as e:
            print(e)

    def mis_total_check(self, mis_check):
        misalignment_output_res = []
        try:

            if mis_check is not None:
                if mis_check == 'misalignment':
                    Misalignment_Check = "14"
                    misalignment_output_res.append(Misalignment_Check)

                if mis_check == 'severe misalignment':
                    Misalignment_Check = "14"
                    misalignment_output_res.append(Misalignment_Check)
                if mis_check == 'No misalignment':
                    Misalignment_Check = "13"
                    misalignment_output_res.append(Misalignment_Check)

                return misalignment_output_res
            else:
                return
        except Exception as e:
            print(e)

    def mis_total_axis_check(self):
        try:
            max_axial_amp, max_radial_amp = self.max_val()
            axial_misalignment_check = self.Misalignment_Check(max_axial_amp)
            radial_misalignment_check = self.Misalignment_Check(max_radial_amp)
            mis_axial_total_check = self.mis_total_check(
                axial_misalignment_check)
            mis_radial_total_check = self.mis_total_check(
                radial_misalignment_check)

            if mis_axial_total_check == "14" or mis_radial_total_check == "14":
                misalignment_res = "14"
            else:
                misalignment_res = "13"
            data = [self.Equipment_id, self.Sensor_id, "F", json.dumps(
                misalignment_res) + "$T$" + str(time.time())]
            v_helper.helper.data_to_publish.append(data)
            return misalignment_res
        except Exception as e:
            print(e)



import numpy as np
import pandas as pd
import math
from numpy.linalg import norm
from numpy import (dot, arccos, clip)
import logers as log
import logging
import vanalytics_helper as v_helper
import util as fft_cal
import rpm as Shaftspeed
import vmaint_constants as constants
import json
import time


class unbalance(object):

    log = log.customLogger(logging.DEBUG)

    def __init__(self, sampling_freq, noOfSample, windowsize, operating_rpm, sensor_id,
                 equipment_id, avg_fftx,avg_ffty,avg_fftz,rms):
        '''
        sampling_freq : Sampling Frequency of the sensor
        noOfSample : Number of samples in FFT, Accelerometer X and Accelerometer Y
        windowsize : window size of the fft data
        shaftSpeed : Maximum speed of the Motor
        fftlist : FFT X-axes Data
        END_THRESHOLD = Amplitude range 1.2 times  (float(1.2))
        SECONDS = 60 seconds
        ANGLE_IN_DEGREE = Complete Angle (360 deg)
        IDEAL_DEGREE = Ideal phase difference
        DEGREE_RANGE = Limit for the Phase difference(+-30 deg)
        START_FREQ_LIMIT = start of frequency range for 1st Harmonic
        END_FREQ_LIMIT = End of frequency range for 1st Harmonic
        LoosenessRange = Harmonics for the other faults
        HARMONIC_ALLOWANCE = At each harmonics frequency limit is 0.2
        MAX_AMP_LIMIT_1X = Amplitude 40 percent at 1st Harmonic
        HARMONIC_END = Harmonics (2X to 10X)
        HARMONICS_RANGE = Harmonics range (0.5X, 1X, 1.5X, 2X,....10.5X)
        START_RANGE = 0
        TWO = 2
        START = 0.0
        END = 1.0
        INI_WINDOWSIZE = 0
        '''
        self.Sampling_Frequency = sampling_freq
        self.Number_sample = noOfSample
        self.Window_Size = windowsize
        self.operating_rpm = operating_rpm
        self.Sensor_id = sensor_id
        self.Equipment_id = equipment_id
        self.avg_fftx=avg_fftx
        self.avg_ffty=avg_ffty
        self.avg_fftz=avg_fftz

        self.window_val = self.Window_Size
        self.HarmonicsStart = []
        self.HarmonicsEnd = []
        self.Harmonic_Frequency = []
        self.rms = rms
        self.rms_threshold = constants.THREE

    def unbalance_axes(self,avg_fftdata):
        try:

            if self.operating_rpm is not None:
                time = 1 / self.Sampling_Frequency
                # FFT Frequencies
                fft_x = np.linspace(int(constants.START), int(constants.END) /
                                    (constants.TWO * time), self.Window_Size)
                fft_x_list = list(fft_x)
                
                fft_rms_index = [index for index, value in enumerate(avg_fftdata)
                                 if value > constants.END_THRESHOLD * self.rms]

                fft_rms_values = [value for index, value in enumerate(avg_fftdata)
                                  if value > constants.END_THRESHOLD * self.rms]
                frequency_above_rms = [fft_x_list[fft_rms_index[i]]
                                       for i in range(len(fft_rms_index))]
                # # Taking rpm 1x range to detect unbalance fault
                rpm1xranage = [frequency_above_rms[i] for i in range(len(frequency_above_rms))
                               if constants.START_FREQ_LIMIT * self.operating_rpm <= frequency_above_rms[i] <= constants.END_FREQ_LIMIT * self.operating_rpm]
                #            # Index of the freq above rms value
                #            indexFreqAboveRMS = [frequency_above_rms[rpm1xranage[i]]
                #                                        for i in range(len(rpm1xranage))]
                indexFreqAboveRMS = frequency_above_rms.index(rpm1xranage[0])

                # Amplitude of the freq
                max1xrpmAmplitude = fft_rms_values[indexFreqAboveRMS]

                # amplitudes and corresponding freq above the threshold and max amplitude in the list
                #            return RPM#fft_rms_values, frequency_above_rms, max1xrpmAmplitude
                return fft_rms_values, frequency_above_rms, max1xrpmAmplitude
                
            else:
                return
        except:
            self.log.error(
                "Exception occurred Frequency above Threshold is not in the range of rpm", exc_info=True)

    def unbalance_harmonics_check(self):

        values_x = self.unbalance_axes(self.avg_fftx)
        values_y = self.unbalance_axes(self.avg_ffty)
        values_z = self.unbalance_axes(self.avg_fftz)
        

        if values_x is not None:
            fft_rms_values = values_x[0]
            frequency_above_rms = values_x[1]
            max1xrpmAmplitude = values_x[2]
        else:
            return
        # harmonicsfrequ = self.unbalance_harmonics_check()
        #rpm = int(self.RpmFromVib()[0])
        try:
            if self.operating_rpm is not None:

                harmonics = np.arange(int(constants.START), int(
                    constants.HARMONIC_END), float(constants.HARMONICS_RANGE))

                for j in harmonics:
                    # print(j)
                    Start_val = (
                        (self.operating_rpm * j) - (self.operating_rpm * float(constants.HARMONIC_ALLOWANCE)))
                    End_val = (
                        (self.operating_rpm * j) + (self.operating_rpm * float(constants.HARMONIC_ALLOWANCE)))
                    self.HarmonicsStart.append(Start_val)
                    self.HarmonicsEnd.append(End_val)

                k = list(zip(self.HarmonicsStart, self.HarmonicsEnd))
                # Checking for multiple harmonics with 20%, from 2x to 10.5x
                # and checking harmonics corresponding to 1x harmonic
                for harmonic_freq_values in frequency_above_rms:
                    for i, j in k:
                        if i <= harmonic_freq_values <= j:
                            self.Harmonic_Frequency.append(
                                harmonic_freq_values)
                        else:
                            pass
                if self.Harmonic_Frequency is not None:
                    harmonicAmpli = [fft_rms_values[frequency_above_rms.index(
                        i)] for i in self.Harmonic_Frequency]
                
                    # Considering harmonics above 40% of 1x harmonics to detect other faults
                    HrmAmpAbv40_per1x = [i for i in harmonicAmpli if i >
                                        float(constants.MAX_AMP_LIMIT_1X) * max1xrpmAmplitude]   
                else:
                    HrmAmpAbv40_per1x=0
                return HrmAmpAbv40_per1x  
            else:
                return
        # return HrmAmpAbv40_per1x
        except:
            self.log.error("Exception occurred while checking amplitudes at multiple Harmonics",
                           exc_info=True)

    def unbalance_total_check(self):
        unbalance_output_res = []
        try:
            AmplitueAbove40Percent1XHarmonic = self.unbalance_harmonics_check()
            # Returning faults with respect to their condition
            # Check for other faults
            if AmplitueAbove40Percent1XHarmonic == 1:
                Unbalance_result = "10"
                unbalance_output_res.append(Unbalance_result)

            # There's no Unbalance Fault
            else:
                Unbalance_result = "9"
                unbalance_output_res.append(Unbalance_result)

            data = [self.Equipment_id, self.Sensor_id, "F", json.dumps(unbalance_output_res) + "$T$" + str(time.time())]
            
            # Stop individual data publish
            v_helper.helper.data_to_publish.append(data)

            return unbalance_output_res
        except:
            self.log.error(
                "Exception occurred while checking Unbalance result", exc_info=True)





import numpy as np
import pandas as pd
# import enum
import vanalytics_helper as v_helper
import logers as log
import logging
import util2 as fft_cal

import vmaint_constants as constants
import json
import time


class Looseness_Analysis(object):

    log = log.customLogger(logging.DEBUG)

    '''
    This is a class to analyse looseness
    '''

    def __init__(self, samplingFrequency, noOfSample, windowsize, operating_rpm_X,
                 sensor_id, equipment_id, avgfft_data_X, avgfft_data_Y, avgfft_data_Z, freqn_X, freqn_Y, freqn_Z):
        '''
        samplingFrequency, windowsize : from sensor details (metadata)
        noOfSample : Number of sampales
        fftlist : FFT data from mqtt
        FREQUENCY_LIMIT = float(0.20)
        RMS_2X_THRESHOLD = float(0.40)
        RMS_3X_THRESHOLD = float(0.30)
        '''
        self.sampling_frequency = samplingFrequency
        self.number_sample = noOfSample
        self.window_size = windowsize
        self.operating_rpm_X = operating_rpm_X
        self.hrm=[]
        self.Equipment_id = equipment_id
        self.Sensor_id = sensor_id
        self.avgfft_data_X = avgfft_data_X
        self.avgfft_data_Y = avgfft_data_Y
        self.avgfft_data_Z = avgfft_data_Z
        self.Limits_of_harmonics = []
        self.sub_limits = []
        self.limit_low = []
        self.limit_high = []
        self.limit_max_values = []
        self.limit2 = []
        self.freqn_X = freqn_X
        self.freqn_Y = freqn_Y
        self.freqn_Z = freqn_Z
        self.amp_data = []

    def Start_End_Array(self, fftdata, freqn, operating_rpm,limit):
        # Shaft Rotational Speed
        try:
            if operating_rpm is not None:
                #FFT_DF = self.avgfft_data_radial
                #limit = operating_rpm *35
                for j in range(1,11):
                    Rpm = (operating_rpm)*j
                    lower_limit, upper_limit = Rpm - limit, Rpm + limit 
                    harmonic_value = [idx for idx, element in enumerate( freqn) if lower_limit <= element <= upper_limit]
                    self.hrm.append(harmonic_value)
                    amplitude_data = [fftdata[i] for i in harmonic_value]
                    max_value_1 = max(amplitude_data)
                    self.limit2.append(max_value_1)
                #print(self.hrm)
                return self.limit2,amplitude_data
                # return self.amp_data

            else:
                return
        except Exception as e:
            print(e)
    def amp_call(self,operatingRpm):
        try:
            limit1=operatingRpm*constants.THIRTY_FIVE_PERCENT
            limit2=operatingRpm*constants.FORTY_PERCENT
            if self.number_sample==8192 and self.window_size==8192:
                max_ampx,ampx = self.Start_End_Array(
                    self.avgfft_data_X, self.freqn_X, self.operating_rpm_X,limit1)
                max_ampy,ampy = self.Start_End_Array(
                    self.avgfft_data_Y, self.freqn_Y, self.operating_rpm_X,limit1)
                max_ampz,ampz = self.Start_End_Array(
                    self.avgfft_data_Z, self.freqn_Z, self.operating_rpm_X,limit1)
            else:
                max_ampx,ampx = self.Start_End_Array(
                    self.avgfft_data_X, self.freqn_X, self.operating_rpm_X,limit2)
                max_ampy,ampy = self.Start_End_Array(
                    self.avgfft_data_Y, self.freqn_Y, self.operating_rpm_X,limit2)
                max_ampz,ampz = self.Start_End_Array(
                    self.avgfft_data_Z, self.freqn_Z, self.operating_rpm_X,limit2)
            return max_ampx,ampx
        except Exception as e:
            print(e)
    def Looseness_Check(self):

        try:
            max_amplitude_at_harmonics_x, amp_x_1x_10x = self.amp_call(self.operating_rpm_X)
            max_amp1x = max_amplitude_at_harmonics_x[0]
            amp_2x_6x = max_amplitude_at_harmonics_x[1:]
            #print(newlist_amp)
            max_amp_1x_120 = constants.ONE_TWENTY_PERCENT * max_amp1x
            # max_amp_1x_120 = 1.5 * max_amp1x
            faultCounter = 0
            if amp_2x_6x is not None:
                for i in amp_2x_6x:
                    if i > max_amp_1x_120:
                        faultCounter = faultCounter+1
                    if faultCounter >= constants.TWO:
                        result = str('Looseness')
                        break
                    else:
                        result = str('No Looseness')

                return result

            else:
                return

        except Exception as e:
            print(e)

    def Looseness_Total_Check(self):
        looseness_output_res = []
        try:
            Looseness_Check = self.Looseness_Check()
            if Looseness_Check is not None:
                if Looseness_Check == 'Looseness':

                    Looseness_Check = '14'
                    looseness_output_res.append(Looseness_Check)

                else:
                    Looseness_Check = '13'
                    
                    looseness_output_res.append(Looseness_Check)

                data = [self.Equipment_id, self.Sensor_id, "F", json.dumps(
                    looseness_output_res) + "$T$" + str(time.time())]
                
                # Stop individual data publish
                # v_helper.helper.data_to_publish.append(data)

                return looseness_output_res
            else:
                return
        except Exception as e:

            print(e)



# imported libraries
import logging
import logers as log_file
import itertools
import numpy as np
import vanalytics_helper as v_helper
import pandas as pd
import time
import json

# MachineStatus is class is called whenever the on of condition of machine is required


class MachineStatus (object):
    log = log_file.customLogger(logging.DEBUG)

    # fftdata, samplingfrequency, equipment_id, sensor_id are input to the class taken from templist,metadata and equipmentdata
    def __init__(self, rmsdata, samplingfrequency, equipment_id, sensor_id):
        # threshold is set by observing penya data, it is not generic yet
        self.threshold = float(0.001)
        self.rmsdata = rmsdata
        self.samplingfrequency = samplingfrequency
        self.equipment_id = equipment_id
        self.sensor_id = sensor_id

    # Below method checks machine is on or off based on threshold declared above
    def check_machine_status(self):
        rms = self.rmsdata
        #  rms and threshold are compared belows
        if rms >= self.threshold:
            machine_status = 1
        else:
            machine_status = 0
        # data is sent for publishing
        data = [self.equipment_id, self.sensor_id, "O", json.dumps(machine_status) + "$T$" + str(time.time())]        
        v_helper.helper.data_to_publish.append(data)        
        return machine_status




import numpy as np
import pandas as pd
# import enum
import vanalytics_helper as v_helper
import logers as log
import logging
import util2 as fft_cal

import vmaint_constants as constants
import json
import time


class Looseness_Analysis(object):

    log = log.customLogger(logging.DEBUG)

    '''
    This is a class to analyse looseness
    '''

    def __init__(self, samplingFrequency, noOfSample, windowsize, operating_rpm,
                 sensor_id, equipment_id, avgfft_data, freqn):
        '''
        samplingFrequency, windowsize : from sensor details (metadata)
        noOfSample : Number of sampales
        fftlist : FFT data from mqtt
        FREQUENCY_LIMIT = float(0.20)
        RMS_2X_THRESHOLD = float(0.40)
        RMS_3X_THRESHOLD = float(0.30)
        '''
        self.sampling_frequency = samplingFrequency
        self.number_sample = noOfSample
        self.window_size = windowsize
        self.operating_rpm = operating_rpm
        self.Equipment_id = equipment_id
        self.Sensor_id = sensor_id
        self.avgfft_data = avgfft_data
        self.Limits_of_harmonics = []
        self.sub_limits = []
        self.limit_low = []
        self.limit_high = []
        self.limit_max_values = []
        self.limit2 = []
        self.freqn = freqn
        self.amp_data = []

    def Start_End_Array(self):
        # Shaft Rotational Speed
        try:
            if self.operating_rpm is not None:
                range1_limit=self.operating_rpm*constants.THIRTY_PERCENT
                for j in range(1, 7):
                    Rpm = (self.operating_rpm)*j
                    range_of_harmonics=Rpm-range1_limit,Rpm+range1_limit
                    self.Limits_of_harmonics.append(range_of_harmonics)
                values = pd.DataFrame(self.Limits_of_harmonics, columns=['start_limit', 'end_limit'])
                return values  
        except Exception as e:
            print(e)

    def Maximum_value_at_harmonics(self):
        try:
            if self.Start_End_Array() is not None:
                # range values at corresponding harmonics
                limit_range = self.Start_End_Array()

                harmonics = limit_range.iloc[[0, 1, 2, 3, 4, 5]]
                #print(harmonics)

                for i in range(6):

                    amp_low = int(harmonics.iloc[i, 0])
                    #amp_low.tolist()
                    amp_high = int(harmonics.iloc[i, 1])
                    self.limit_low.append(amp_low)

                    self.limit_high.append(amp_high)
                    #self.limit_low=[round(num, 2) for num in self.limit_low]
                    #self.limit_high = [round(num, 2) for num in self.limit_high]
                df1 = pd.DataFrame(self.limit_low)
                df2 = pd.DataFrame(self.limit_high)

                return df1, df2

        except Exception as e:
            print(e)

    def max_val(self):
        try:
            if self.avgfft_data is not None:
                # fft_data = self.RpmFromVib()[1]
                FFT_DF = self.avgfft_data
                data1, data2 = self.Maximum_value_at_harmonics()
                for j in range(6):
                    amp_low2 = int(data1.iloc[j, 0])
                    amp_high2 = int(data2.iloc[j, 0])
                    #prxint(FFT_DF)
                    #print(type(self.freqn))
                    #freqndata=np.array(self.freqn)

                    data = [idx for idx, element in enumerate(
                        self.freqn) if amp_low2 <= element <= amp_high2]

                    amplitude_data = [FFT_DF[i] for i in data]
                    
                    max_value_1 = max(amplitude_data)
                    #print(max_value_1)
                    self.limit2.append(max_value_1)

                return self.limit2
            else:
                return
        except Exception as e:
            print(e)
    def Looseness_Check(self):

        try:
            max_amplitude_at_harmonics = self.max_val()
            max_amp1x = max_amplitude_at_harmonics[0]
            amp_2x_6x = max_amplitude_at_harmonics[1:]
            #print(newlist_amp)
            max_amp_1x_120 = constants.ONE_TWENTY_PERCENT*max_amp1x
            faultCounter = 0
            if amp_2x_6x is not None:
                for i in amp_2x_6x:
                    if i > max_amp_1x_120:
                        faultCounter = faultCounter+1
                    if faultCounter >= constants.TWO:
                        result = str('Looseness')
                        break
                    else:
                        result = str('No Looseness')

                return result

            else:
                return

        except Exception as e:
            print(e)

    def Looseness_Total_Check(self):
        looseness_output_res = []
        try:
            Looseness_Check = self.Looseness_Check()
            if Looseness_Check is not None:
                if Looseness_Check == 'Looseness':
                    Looseness_Check = '14'
                    los_result = 'Looseness'
                    looseness_output_res.append(los_result)

                else:
                    Looseness_Check = '13'
                    los_result = 'no Looseness'
                    looseness_output_res.append(los_result)

                data = [self.Equipment_id, self.Sensor_id, "F", json.dumps(
                    looseness_output_res) + "$T$" + str(time.time())]
                v_helper.helper.data_to_publish.append(data)

                return looseness_output_res
            else:
                return
        except Exception as e:

            print(e)



import numpy as np
import pandas as pd
import math
from numpy.linalg import norm
from numpy import (dot, arccos, clip)
import logers as log
import logging
import vanalytics_helper as v_helper
import util as fft_cal
import rpm as Shaftspeed
import vmaint_constants as constants
import json
import time


class unbalance(object):

    log = log.customLogger(logging.DEBUG)

    def __init__(self, sampling_freq, noOfSample, windowsize, operating_rpm, sensor_id,
                 equipment_id, horizontAccelData, verticalAccelData, avg_fft, rms):
        '''
        sampling_freq : Sampling Frequency of the sensor
        noOfSample : Number of samples in FFT, Accelerometer X and Accelerometer Y
        windowsize : window size of the fft data
        shaftSpeed : Maximum speed of the Motor
        horizontAccelData : Accelerometer X-axes data
        verticalAccelData : Accelerometer Y-axes data
        fftlist : FFT X-axes Data
        END_THRESHOLD = Amplitude range 1.2 times  (float(1.2))
        SECONDS = 60 seconds
        ANGLE_IN_DEGREE = Complete Angle (360 deg)
        IDEAL_DEGREE = Ideal phase difference
        DEGREE_RANGE = Limit for the Phase difference(+-30 deg)
        START_FREQ_LIMIT = start of frequency range for 1st Harmonic
        END_FREQ_LIMIT = End of frequency range for 1st Harmonic
        LoosenessRange = Harmonics for the other faults
        HARMONIC_ALLOWANCE = At each harmonics frequency limit is 0.2
        MAX_AMP_LIMIT_1X = Amplitude 40 percent at 1st Harmonic
        HARMONIC_END = Harmonics (2X to 10X)
        HARMONICS_RANGE = Harmonics range (0.5X, 1X, 1.5X, 2X,....10.5X)
        START_RANGE = 0
        TWO = 2
        START = 0.0
        END = 1.0
        INI_WINDOWSIZE = 0
        '''
        self.Sampling_Frequency = sampling_freq
        self.Number_sample = noOfSample
        self.Window_Size = windowsize
        self.operating_rpm = operating_rpm
        self.Sensor_id = sensor_id
        self.Equipment_id = equipment_id
        self.dataframe_X = horizontAccelData
        self.dataframe_Y = verticalAccelData
        self.avgfft_data = avg_fft
        self.window_val = self.Window_Size
        self.HarmonicsStart = []
        self.HarmonicsEnd = []
        self.Harmonic_Frequency = []
        self.rms = rms
        self.rms_threshold = constants.THREE

    def unbalance_axes(self):
        try:

            if self.operating_rpm is not None:
                time = 1 / self.Sampling_Frequency
                # FFT Frequencies
                fft_x = np.linspace(int(constants.START), int(constants.END) /
                                    (constants.TWO * time), self.Window_Size)
                fft_x_list = list(fft_x)
                # FFT Averaged Data
                #fft_y_list = self.RpmFromVib()[1]
                # RMS of FFT Data
                #fft_rms_Amp = self.RpmFromVib()[2]
                # Taking only those data which is above 1.2 times fft rms
                # taking amplitude above 1.5*rms
                fft_rms_index = [index for index, value in enumerate(self.avgfft_data)
                                 if value > constants.END_THRESHOLD * self.rms]

                fft_rms_values = [value for index, value in enumerate(self.avgfft_data)
                                  if value > constants.END_THRESHOLD * self.rms]
                frequency_above_rms = [fft_x_list[fft_rms_index[i]]
                                       for i in range(len(fft_rms_index))]
                # # Taking rpm 1x range to detect unbalance fault
                rpm1xranage = [frequency_above_rms[i] for i in range(len(frequency_above_rms))
                               if constants.START_FREQ_LIMIT * self.operating_rpm <= frequency_above_rms[i] <= constants.END_FREQ_LIMIT * self.operating_rpm]
                #            # Index of the freq above rms value
                #            indexFreqAboveRMS = [frequency_above_rms[rpm1xranage[i]]
                #                                        for i in range(len(rpm1xranage))]
                indexFreqAboveRMS = frequency_above_rms.index(rpm1xranage[0])

                # Amplitude of the freq
                max1xrpmAmplitude = fft_rms_values[indexFreqAboveRMS]

                # amplitudes and corresponding freq above the threshold and max amplitude in the list
                #            return RPM#fft_rms_values, frequency_above_rms, max1xrpmAmplitude
                return fft_rms_values, frequency_above_rms, max1xrpmAmplitude
            else:
                return
        except:
            self.log.error(
                "Exception occurred while checking Frequency above Threshold", exc_info=True)

    def unbalance_harmonics_check(self):

        values = self.unbalance_axes()
        if values is not None:
            fft_rms_values = values[0]
            frequency_above_rms = values[1]
            max1xrpmAmplitude = values[2]
        else:
            return
        # harmonicsfrequ = self.unbalance_harmonics_check()
        #rpm = int(self.RpmFromVib()[0])
        try:
            if self.operating_rpm is not None:

                harmonics = np.arange(int(constants.START), int(
                    constants.HARMONIC_END), float(constants.HARMONICS_RANGE))

                for j in harmonics:
                    # print(j)
                    Start_val = (
                        (self.operating_rpm * j) - (self.operating_rpm * float(constants.HARMONIC_ALLOWANCE)))
                    End_val = (
                        (self.operating_rpm * j) + (self.operating_rpm * float(constants.HARMONIC_ALLOWANCE)))
                    self.HarmonicsStart.append(Start_val)
                    self.HarmonicsEnd.append(End_val)

                k = list(zip(self.HarmonicsStart, self.HarmonicsEnd))
                # Checking for multiple harmonics with 20%, from 2x to 10.5x
                # and checking harmonics corresponding to 1x harmonic
                for harmonic_freq_values in frequency_above_rms:
                    for i, j in k:
                        if i <= harmonic_freq_values <= j:
                            self.Harmonic_Frequency.append(
                                harmonic_freq_values)
                        else:
                            pass
                harmonicAmpli = [fft_rms_values[frequency_above_rms.index(
                    i)] for i in self.Harmonic_Frequency]
                # Considering harmonics above 40% of 1x harmonics to detect other faults
                HrmAmpAbv40_per1x = [i for i in harmonicAmpli if i >
                                     float(constants.MAX_AMP_LIMIT_1X) * max1xrpmAmplitude]
                return HrmAmpAbv40_per1x
            else:
                return
        # return HrmAmpAbv40_per1x
        except:
            self.log.error("Exception occurred while checking amplitudes at multiple Harmonics",
                           exc_info=True)

    def accelerometer_data(self):

        try:
            if (self.dataframe_X is not None) and (self.dataframe_Y is not None):
                # Accelerometer X-axes Data
                amplitudeX = pd.DataFrame(self.dataframe_X)
                # converting string to numeric
                amplitudeX = amplitudeX.apply(pd.to_numeric, errors='coerce')
                x = amplitudeX[0]
                # Accelerometer Y-axes Data
                amplitudeY = pd.DataFrame(self.dataframe_Y)
                # converting string to numeric
                amplitudeY = amplitudeY.apply(pd.to_numeric, errors='coerce')
                y = amplitudeY[0]
                return x, y
            else:
                return
        except Exception as e:
            self.log.error(
                "Exception occurred while checking Unbalance acc data", exc_info=True)
            print(e)

    def phasefind(self):
        try:
            horizontalaxesamplitude, verticalamplitude = self.accelerometer_data()
            if (horizontalaxesamplitude is not None) and (verticalamplitude is not None):
                if len(horizontalaxesamplitude) != len(verticalamplitude):
                    # Absolute value of difference
                    difference = abs(
                        len(horizontalaxesamplitude) - len(verticalamplitude))
                    diff = -(int(difference))
                    # If the lenth of Acc_X-axes is greater than Acc_Y-axes
                    if len(horizontalaxesamplitude) > len(verticalamplitude):
                        # Eliminate the noOfdifference in Acceleromer X-axes
                        df_horizontal = horizontalaxesamplitude[: diff]
                        df_vertical = verticalamplitude
                    # If the lenth of Acc_Y-axes is greater than Acc_X-axes
                    elif len(verticalamplitude) > len(horizontalaxesamplitude):
                        # Eliminate the noOfdifference in Acceleromer Y-axes
                        df_vertical = verticalamplitude[: diff]
                        df_horizontal = horizontalaxesamplitude
                    # Data for calculating Phase Difference
                    amplitudeX = df_horizontal
                    amplitudeY = df_vertical
                    c = dot(amplitudeX, amplitudeY) / \
                        norm(amplitudeX)/norm(amplitudeY)
                    # Phase Difference in Radians
                    angleinradians = arccos(clip(c, -1, 1))
                    # Phase Difference in degrees
                    angleindegree = (
                        (angleinradians * int(constants.ANGLE_IN_DEGREE)) / int(constants.TWO * math.pi))
                else:
                    amplitudeX = horizontalaxesamplitude
                    amplitudeY = verticalamplitude
                    c = dot(amplitudeX, amplitudeY) / \
                        norm(amplitudeX)/norm(amplitudeY)
                    # Phase Difference in Radians
                    angleinradians = arccos(clip(c, -1, 1))
                    # Phase Difference in degrees
                    angleindegree = (
                        (angleinradians * int(constants.ANGLE_IN_DEGREE)) / (int(constants.TWO) * math.pi))
                return angleindegree
            else:
                return
        except Exception as e:
            print(e)

    def unbalance_total_check(self):
        unbalance_output_res = []
        try:
            # For phase angle range
            start_angle = int(constants.IDEAL_DEGREE - constants.DEGREE_RANGE)
            end_angle = int(constants.IDEAL_DEGREE + constants.DEGREE_RANGE)
            Angle = self.phasefind()
            AmplitueAbove40Percent1XHarmonic = self.unbalance_harmonics_check()
            # Returning faults with respect to their condition
            # Check for other faults
            if AmplitueAbove40Percent1XHarmonic != []:
                Unbalance_result = "9"
                unbalance_output_res.append(Unbalance_result)

            # Unbalance phase angle should be 90 degree with allowance(+- 30 deg)
            elif start_angle <= Angle <= end_angle:
                Unbalance_result = "10"
                unbalance_output_res.append(Unbalance_result)

            # There's no Unbalance Fault
            else:
                Unbalance_result = "9"
                unbalance_output_res.append(Unbalance_result)

            # data = [self.Equipment_id, self.Sensor_id, "F", json.dumps(
            #     unbalance_output_res) + "$T$" + str(time.time())]
            # v_helper.helper.data_to_publish.append(data)

            return unbalance_output_res
        except:
            self.log.error(
                "Exception occurred while checking Unbalance result", exc_info=True)



import numpy as np
import pandas as pd
import vanalytics_helper as v_helper
import logers as log
import logging
import util as fft_cal
import rpm as Shaftspeed
import json
import vmaint_constants as constants
import time


class Misalignment_Analysis(object):

    log = log.customLogger(logging.DEBUG)

    
    """This is a class to analyse misalignment"""

    def __init__(self, samplingFrequency, noOfSample, windowsize, operating_rpm,
                 sensor_id, equipment_id, avgfft_data,rms):
        '''
        samplingFrequency, windowsize : from sensor details (metadata)
        noOfSample : Number of sampales
        fftlist : FFT data from mqtt
        FREQUENCY_LIMIT = float(0.20)
        RMS_2X_THRESHOLD = float(0.40)
        RMS_3X_THRESHOLD = float(0.30)
        '''
        self.sampling_frequency = samplingFrequency
        self.number_sample = noOfSample
        self.window_size = windowsize
        self.operating_rpm = operating_rpm
        self.Equipment_id = equipment_id
        self.Sensor_id = sensor_id
        self.avgfft_data = avgfft_data
        self.Limits_of_harmonics = []
        self.sub_limits = []
        self.rms_threshold = constants.THREE
        self.rms=float(rms)
    
    def Start_End_Array(self):
        try:
            if self.operating_rpm is not None:
                for i in range(int(constants.END), constants.HARM_END_RNG):
                    Rpm = self.operating_rpm * i
                    limits = Rpm - (constants.FREQUENCY_LIMIT *
                                    Rpm), Rpm + (constants.FREQUENCY_LIMIT * Rpm)
                    self.sub_limits.append(limits)
                    
                return self.sub_limits
        except:
            self.log.error(
                "Exception occurred while checking limits for the harmonics", exc_info=True)


class limits_of_harmonics(Misalignment_Analysis):

    def Rms_Ranges(self):

        try:
            # RMS calculated from FFT data
            # RMS at 1X Harmonic
            
            if self.rms is not None:
                # RMS at 2X Harmonic
                self.rms_2 = self.rms * constants.RMS_2X_THRESHOLD
                # RMS at 3X Harmonic
                self.rms_3 = self.rms * constants.RMS_3X_THRESHOLD
                rms = [self.rms, self.rms_2, self.rms_3]
                
                return rms
            else:
                return
        except:
            self.log.error("Exception occurred while checking RMS values at 1X, 2X and 3X harmonics",
                           exc_info=True)
#        return rms

    def Limits_of_Harmonics(self):
        try:
            if self.Start_End_Array() is not None:
                for j in self.Start_End_Array():
                    self.Limits_of_harmonics.append(j)

                values = pd.DataFrame(self.Limits_of_harmonics, columns=[
                                      'start_limit', 'end_limit'])
                return values
            else:
                return
        except:
            self.log.error(
                "Exception occurred while checking Limit ranges", exc_info=True)

    def Maximum_value_at_harmonics(self):
        try:
            if self.Limits_of_Harmonics() is not None:
                # range values at corresponding harmonics
                limit_range = self.Limits_of_Harmonics()
                harmonics = limit_range.iloc[[0, 1, 2]]
                amp_one = int(harmonics.iloc[0, 0])
                amp_two = int(harmonics.iloc[0, 1])
                amp_three = int(harmonics.iloc[1, 0])
                amp_four = int(harmonics.iloc[1, 1])
                amp_five = int(harmonics.iloc[2, 0])
                amp_six = int(harmonics.iloc[2, 1])
            else:
                return
            # Frequency range at harmonics 1X, 2X and 3X
            if  self.avgfft_data is not None:
                FFT_DF = pd.DataFrame(self.avgfft_data)
                harmonics_at_1 = FFT_DF.iloc[amp_one: amp_two]
                harmonics_at_2 = FFT_DF.iloc[amp_three: amp_four]
                harmonics_at_3 = FFT_DF.iloc[amp_five: amp_six]
                # Fetching Maximum amplitudes at harmonics 1X, 2X and 3X
                max_value_1X = max(harmonics_at_1[0])
                max_value_2X = max(harmonics_at_2[0])
                max_value_3X = max(harmonics_at_3[0])

                return max_value_1X, max_value_2X, max_value_3X
            else:
                return
        except:
            self.log.error(
                "Exception occurred while checking Maximum value at harmonics", exc_info=True)

    def Misalignment_Check(self):

        try:
            # Maximum Amplitudes at harmonics 1X, 2X, 3X
            max_amplitude_at_harmonics = self.Maximum_value_at_harmonics()
            if max_amplitude_at_harmonics is not None:
                amplitude_1 = float(max_amplitude_at_harmonics[0])
                amplitude_2 = float(max_amplitude_at_harmonics[1])
                amplitude_3 = float(max_amplitude_at_harmonics[2])
            else:
                return
            # RMS calculated from FFT data
            rms = self.Rms_Ranges()
            if rms is not None:
                rms_1 = float(rms[0])
                rms_2 = float(rms[1])
                rms_3 = float(rms[2])
            else:
                return
            # If the max amplitude(amplitude_1) at harmonics 1X greater than RMS(rms_1),
            # then there might be a misalignmnet
            if amplitude_1 >= rms_1:
                result = str("Harmonic at 1x RPM")
                # If the max amplitude at harmonics(amplitude_2) 2X greater than RMS_2(rms_2), then Axial Misalignment
                if amplitude_2 >= rms_2:
                    result = str('Axial Misalignment')

                # If the max amplitude at harmonics 2X greater than max amplitude at harmonics 1X,
                # then Parallel Misalignment
                elif amplitude_2 >= amplitude_1:
                    result = str('ParalleMisalignment')

                    # If the max amplitude(amplitude_3) at harmonics 3X greater than RMS_3(rms_3), then Severe Misalignment
                    if amplitude_3 >= rms_3:
                        result = str('Severe Misalignment')

            else:
                result = str('No Misalignment')
            return result
        except:
            self.log.error(
                "Exception occurred while checking Misalignment Result", exc_info=True)

    def Misalignment_Total_Check(self):

        misalignment_output_res = []
        try:
           # Checking the result
            Misalignment_Check = self.Misalignment_Check()
            if Misalignment_Check is not None:
                if Misalignment_Check == 'Axial_Misalignment':
                    Misalignment_Check = "5"
                    misalignment_output_res.append(Misalignment_Check)
                elif Misalignment_Check == 'Parallel_Misalignment':
                    Misalignment_Check = "6"
                    misalignment_output_res.append(Misalignment_Check)
                elif Misalignment_Check == 'Severe Misalignment':
                    Misalignment_Check = "7"
                    misalignment_output_res.append(Misalignment_Check)
                else:
                    Misalignment_Check = "8"
                    misalignment_output_res.append(Misalignment_Check)

                # F -> "Fault"
                # data = [self.Equipment_id, self.Sensor_id, "F", json.dumps(
                #     misalignment_output_res) + "$T$" + str(time.time())]
                # v_helper.helper.data_to_publish.append(data)
                
                return misalignment_output_res
            else:
                return None
        except:
            self.log.error(
                "Exception occurred while checking Misalignment Analysis Status", exc_info=True)



import json
import numpy as np
import pandas as pd
import logers as log
import logging
import vanalytics_helper as v_helper
import vanalytics_data as v_data

class RpmFromVibration():

    log = log.customLogger(logging.DEBUG)

    def __init__(self, sampling_freq, windowsize, shaftSpeed, sensor_id,
                 equipment_id, fftlist):
        '''
        sampling_freq : Sampling Frequency of the sensor
        noOfSample : Number of samples in FFT, Accelerometer X and Accelerometer Y
        windowsize : window size of the fft data
        shaftSpeed : Maximum speed of the Motor
        fftlist    : FFT Data
        FreqInHzConv : To convert RPM into Hz
        thresholdStart : Amplitude range starts at 0.3 times frequency  (float(1.2))
        thresholdend : Amplitude range ends at 1.2 times frequency  (float(1.2))
        FFTStartRng : FFT Start Range
        '''
        self.Sampling_Frequency = sampling_freq
        self.Window_Size = windowsize
        self.MaximumSpeed = shaftSpeed
        self.sensor_id = sensor_id
        self.equipment_id = equipment_id
        self.FFT_list = fftlist
        self.thresholdStart = float(2.3)
        self.thresholdend = float(2.8)
        self.start = 0.0
        self.end = 1.0
        self.FreqInHzConv = 60
        self.Two = 2
        self.FFTStartRng = 0

    def Speed_Detection(self):
        try:
            time = 1 / self.Sampling_Frequency
            # FFT Amplitudes
            # fft_y = np.array(self.FFT_list[self.FFTStartRng : self.Window_Size])
            # FFT Frequencies            
            fft_x = np.linspace(self.start, self.end /
                                (self.Two * time), self.Window_Size//2)
            
            fft_frequencies = list(fft_x)
            fft_amplitudes = self.FFT_list
            # Converting ShaftSpeed from rev/min to Hertz (Hz)
            maximum_rpm_hz = self.MaximumSpeed / self.FreqInHzConv
            # Frequency Start Limit
            rpm_thresholdStart = self.thresholdStart * maximum_rpm_hz
            # Frequency End Limit
            rpm_thresholdend = self.thresholdend * maximum_rpm_hz

            # Indexes which is between than Frequency Limits
            indexfreqencylesthanrpm = [fft_amplitudes[k] for k in range(
                len(fft_frequencies)) if rpm_thresholdStart <= fft_frequencies[k] <= rpm_thresholdend]

            # Maximum Amplitude among the list in amplitudesrpm
            maxoneamplitude = max(indexfreqencylesthanrpm)
            # Index of the Maximum Amplitude
            indexofamplitude = fft_amplitudes.index(maxoneamplitude)
            # Frequency corresponding to the Index
            freuencycorrestomaxampl = fft_frequencies[indexofamplitude]
            # Int of the Speed (Hz)
            RPM_from_Vib = round(freuencycorrestomaxampl/3, 2)

            data = [self.equipment_id, self.sensor_id, "R", RPM_from_Vib]
            v_helper.helper.data_to_publish.append(data)

            self.log.info("Calculating Shaft Speed From FFT Data")
            if v_data.CommonFeatures.simulator_rpm > 0:
                return v_data.CommonFeatures.simulator_rpm 
                #round((v_data.CommonFeatures.simulator_rpm / 60), 2)
            else:
                return RPM_from_Vib
        except:
            self.log.error(
                "Exception occurred while Calculating Shaft Speed From FFT Data", exc_info=True)



import paho.mqtt.client as mqtt
import time
import logging
import threading
import logers
import numpy as np
import vanalytics_helper as v_helper
import vanalytics_data as v_data


class MqttClient(object):
    log = logers.customLogger(logging.DEBUG)

    def __init__(self, ipAddress, portNumber, username=None, password=None, brokerName=None):
        self.IpAddress = ipAddress
        self.PortNumber = int(portNumber)
        self.Username = username
        self.Password = password
        self.BrokerName = brokerName
        self.Keepalive = 1000
        self.Client = None
        self.isConnected = 0

    def on_connect(self, client, userdata, flags, rc):
        if rc == 0:
            # set flag
            client.connected_flag = True
            self.isConnected = 1
            print("connected OK Returned code=", rc)
        else:
            print("Bad connection Returned code=", rc)

    def on_disconnect(self, client, userdata, rc):
        try:
            self.log.info("Client disconnected=%s", rc)
            self.isConnected = 0
            client.loop_stop()
            broker_keys = [self.IpAddress + "_" + str(self.PortNumber)]
            v_data.CommonFeatures.connect_subscribe_mqtt(broker_keys)
        except:
            self.log.error(
                "Exception occurred while disconnecting the broker,Returned code= %s", rc, exc_info=True)

    def on_subscribe(self, client, userdata, mid, granted_qos):
        try:
            pass
        except:
            self.log.critical(
                "Exception occurred while subscribing the topics", exc_info=True)

    def on_unsubscribe(self, client, userdata, mid):
        try:
            pass
        except:
            self.log.critical(
                "Exception occurred while unsubscribing the topics", exc_info=True)

    def on_message(self, client, userdata, message):
        try:
            # print(message)
            data = [message.topic, message.payload.decode("utf-8")]
            
            v_helper.helper.data_queue.put(data)
            # print(v_helper.helper.data_queue.qsize())
        except:
            self.log.error("Exception occurred while receving message %s", str(
                message.topic), exc_info=True)

    def connect_broker(self):
        clientid = "vegam_analytics_" + str(time.time_ns())
        self.Client = mqtt.Client(clientid, False)

        if self.Username is not None and self.Password is not None:
            self.Username = self.Username.strip()
            self.Password = self.Password.strip()

            if len(self.Username) > 0 and len(self.Password):
                self.Client.username_pw_set(self.Username, self.Password)

        self.log.info("Connecting to broker at %s:%s",
                      self.IpAddress, self.PortNumber)
        try:
            self.Client.connect(
                self.IpAddress, self.PortNumber, self.Keepalive)
            self.log.info("Connected")
        except Exception as ex:
            self.log.error('Failed with error: %s', ex, exc_info=True)

        self.Client.on_connect = self.on_connect
        self.Client.on_subscribe = self.on_subscribe
        self.Client.on_disconnect = self.on_disconnect
        self.Client.on_unsubscribe = self.on_unsubscribe
        self.Client.on_message = self.on_message

        self.Client.loop_start()

    def disconnect_broker(self):
        self.Client.loop_stop()
        # TODO: unsubscribe from here

    def subscribe_topic(self, topics, qos=2):
        for topic in topics:
            self.Client.subscribe(topic, qos)

    def publish_data(self, topic, message, qos=2):
        self.Client.publish(topic, message, qos)



# imported libraries
import numpy as np
import logging
import statistics
import vanalytics_helper as v_helper
import itertools
import util as fft
import time
import json
import vmaint_constants as constants
import logers as log_file

class MachineStatus (object):

    log = log_file.customLogger(logging.DEBUG)

    # rmsdata, samplingfrequency, equipment_id, sensor_id are input to the class
    def __init__(self, samplingfrequency, equipment_id, sensor_id, rms_valueX, rms_valueY, rms_valueZ):
        self.rmsthreshold = constants.rmsthreshold
        self.rms_valueX = rms_valueX
        self.rms_valueY = rms_valueY
        self.rms_valueZ = rms_valueZ
        self.samplingfrequency = samplingfrequency
        self.equipment_id = equipment_id
        self.sensor_id = sensor_id

    def avg_three_axis(self):
        try:
            average_total = (self.rms_valueX + self.rms_valueY +
                            self.rms_valueZ) / constants.THREE
            return average_total
        except:
            self.log.critical(
                "Exception occurred while calculating average in onoff", exc_info=True)

    # Below method checks machine is on or off based on threshold declared above
    def check_machine_status(self):
        try:
            average_rms = self.avg_three_axis()
            self.log.info("average_rms value is=",average_rms,"of sensorid=",self.sensor_id,"time=",str(time.time()))
            #   1 machine on & 0 machine off
            if average_rms >= self.rmsthreshold:
                machine_status = 1  
            else:
                machine_status = 0
            
            data = [self.equipment_id, self.sensor_id, "O", json.dumps(machine_status) + "$T$" + str(time.time())]
            
            v_helper.helper.data_to_publish.append(data)
            return machine_status
        except:
            self.log.critical(
                "Exception occurred while checking the machine status", exc_info=True)



# -*- coding: utf-8 -*-
"""
Created on Tue Dec  7 15:19:41 2021

@author: laptop58
"""


import numpy as np
import pandas as pd
# import enum
import vanalytics_helper as v_helper
import logers as log
import logging
import util2 as fft_cal
import json
import time
from numpy.linalg import norm
from numpy import (dot, arccos, clip)
import vmaint_constants as constants
from statistics import mean
import math


class Misalignment_Analysis(object):

    #log = log.customLogger(logging.DEBUG)

    '''
    This is a class to analyse looseness
    '''

    def __init__(self, samplingFrequency, noOfSample, windowsize, operating_rpm_X,
                 sensor_id, equipment_id, avgfft_data_X, avgfft_data_Y, avgfft_data_Z, freqn_X, freqn_Y, freqn_Z):
        '''
        samplingFrequency, windowsize : from sensor details (metadata)
        noOfSample : Number of sampales
        fftlist : FFT data from mqtt
        FREQUENCY_LIMIT = float(0.20)
        RMS_2X_THRESHOLD = float(0.40)
        RMS_3X_THRESHOLD = float(0.30)
        '''
        self.sampling_frequency = samplingFrequency
        self.number_sample = noOfSample
        self.window_size = windowsize
        self.operating_rpm_X = operating_rpm_X
        #self.operating_rpm1 = operating_rpm1
        self.Equipment_id = equipment_id
        self.Sensor_id = sensor_id
        self.avgfft_data_X = avgfft_data_X
        self.avgfft_data_Y = avgfft_data_Y
        self.avgfft_data_Z = avgfft_data_Z
        self.sub_limits = []

        self.freqn_X = freqn_X
        self.freqn_Y = freqn_Y
        self.freqn_Z = freqn_Z
        #self.dataframe_Y=accy
        #self.dataframe_X=accx
        self.amp_data = []
        self.limit2 = []
        self.indexes = []
        self.indexes1 = []
        self.hrm = []

    def Start_End_Array(self, fftdata, freqn, operating_rpm,limit):
        # Shaft Rotational Speed
        try:
            if operating_rpm is not None:
                FFT_DF = fftdata
                #FFT_DF = self.avgfft_data_radial
                #limit = operating_rpm *35
                for j in range(1,11):
                    Rpm = (operating_rpm)*j
                    lower_limit, upper_limit = Rpm - limit, Rpm + limit 
                    harmonic_value = [idx for idx, element in enumerate( freqn) if lower_limit <= element <= upper_limit]
                    #self.hrm.append(harmonic_value)
                    amplitude_data = [FFT_DF[i] for i in harmonic_value]
                    max_value_1 = max(amplitude_data)
                    self.limit2.append(max_value_1)
                
                return self.limit2,amplitude_data
                # return self.amp_data

            else:
                return
        except Exception as e:
            print(e)

    """ def accelerometer_data(self):

        try:
            if (self.dataframe_X is not None) and (self.dataframe_Y is not None):
                # Accelerometer X-axes Data
                amplitudeX = pd.DataFrame(self.dataframe_X)
                # converting string to numeric
                amplitudeX = amplitudeX.apply(pd.to_numeric, errors='coerce')
                x = amplitudeX[0]
                # Accelerometer Y-axes Data
                amplitudeY = pd.DataFrame(self.dataframe_Y)
                # converting string to numeric
                amplitudeY = amplitudeY.apply(pd.to_numeric, errors='coerce')
                y = amplitudeY[0]
                return x, y
            else:
                 return
        except Exception as e:
            print(e)

    def phasefind(self):
        try:
            horizontalaxesamplitude, verticalamplitude = self.accelerometer_data()
            if (horizontalaxesamplitude is not None) and (verticalamplitude is not None):
                if len(horizontalaxesamplitude) != len(verticalamplitude):
                    # Absolute value of difference
                    difference = abs(
                        len(horizontalaxesamplitude) - len(verticalamplitude))
                    diff = -(int(difference))
                    # If the lenth of Acc_X-axes is greater than Acc_Y-axes
                    if len(horizontalaxesamplitude) > len(verticalamplitude):
                        # Eliminate the noOfdifference in Acceleromer X-axes
                        df_horizontal = horizontalaxesamplitude[: diff]
                        df_vertical = verticalamplitude
                    # If the lenth of Acc_Y-axes is greater than Acc_X-axes
                    elif len(verticalamplitude) > len(horizontalaxesamplitude):
                        # Eliminate the noOfdifference in Acceleromer Y-axes
                        df_vertical = verticalamplitude[: diff]
                        df_horizontal = horizontalaxesamplitude
                    # Data for calculating Phase Difference
                    amplitudeX = df_horizontal
                    amplitudeY = df_vertical
                    c = dot(amplitudeX, amplitudeY) / \
                        norm(amplitudeX)/norm(amplitudeY)
                    # Phase Difference in Radians
                    angleinradians = arccos(clip(c, -1, 1))
                    # Phase Difference in degrees
                    angleindegree = (
                        (angleinradians * int(constants.ANGLE_IN_DEGREE)) / int(constants.TWO * math.pi))
                else:
                    amplitudeX = horizontalaxesamplitude
                    amplitudeY = verticalamplitude
                    c = dot(amplitudeX, amplitudeY) / \
                        norm(amplitudeX)/norm(amplitudeY)
                    # Phase Difference in Radians
                    angleinradians = arccos(clip(c, -1, 1))
                    degree=math.degrees(angleinradians)
                    print("radians",angleinradians)
                    print("degree",degree)
                    # Phase Difference in degrees
                    angleindegree = (
                        (angleinradians * int(constants.ANGLE_IN_DEGREE)) / (int(constants.TWO) * math.pi))
                return degree
            else:
                return
        except Exception as e:
            print(e)"""
    def amp_call(self,operatingRpm):
        try:
            limit1=operatingRpm*constants.THIRTY_PERCENT
            limit2=operatingRpm*constants.FORTY_PERCENT
            if self.number_sample==8192 and self.window_size==8192:
                max_ampx,ampx = self.Start_End_Array(
                    self.avgfft_data_X, self.freqn_X, self.operating_rpm_X,limit1)
                max_ampy,ampy = self.Start_End_Array(
                    self.avgfft_data_Y, self.freqn_Y, self.operating_rpm_X,limit1)
                max_ampz,ampz = self.Start_End_Array(
                    self.avgfft_data_Z, self.freqn_Z, self.operating_rpm_X,limit1)
            else:
                max_ampx,ampx = self.Start_End_Array(
                    self.avgfft_data_X, self.freqn_X, self.operating_rpm_X,limit2)
                max_ampy,ampy = self.Start_End_Array(
                    self.avgfft_data_Y, self.freqn_Y, self.operating_rpm_X,limit2)
                max_ampz,ampz = self.Start_End_Array(
                    self.avgfft_data_Z, self.freqn_Z, self.operating_rpm_X,limit2)
            return max_ampx,ampx
        except Exception as e:
            print(e)
    def max_val_data(self, fftdata, freqn, operating_rpm):
        try:
            maxamp_values, amp_values = self.amp_call(self.operating_rpm_X)
            amp_3x_6x = maxamp_values[2:]
            amp_1x_50 = maxamp_values[0]*constants.FIFTY_PERCENT
            amp_1x_150 = maxamp_values[0]*constants.ONE_FIFTY_PERCENT
            amp_2x_50 = maxamp_values[1]*constants.FIFTY_PERCENT

            if amp_1x_50 < maxamp_values[1] <= amp_1x_150:
                result = str('misalignment')
                if maxamp_values[1] > amp_1x_150:
                    result = str('severe misalignment')

            else:
                result = str('No misalignment')

                # else:
                #     result = str('No misalignment')

            return result, amp_values
        except Exception as e:
            print(e)

    def mis_axis_check(self, fftdata, freqn, operating_rpm):

        misalignment_res = []
        try:
            max_value, amp_value = self.max_val_data(
                fftdata, freqn, operating_rpm)
            #mis_result_axial = self.mis_total_check(self.avgfft_data_axial, self.freqn_axial)
            #mis_result_radial = self.mis_total_check(
            #self.avgfft_data_radial, self.freqn_radial)
            #angle_phase = self.phasefind()
            misalignment_output_res = []

            if max_value is not None:
                if max_value == 'misalignment':
                    Misalignment_Check = "7"
                    misalignment_output_res.append(Misalignment_Check)

                if max_value == 'severe misalignment':
                    Misalignment_Check = "7"
                    misalignment_output_res.append(Misalignment_Check)
                if max_value == 'No misalignment':
                    Misalignment_Check = "8"
                    misalignment_output_res.append(Misalignment_Check)
                #print(misalignment_output_res)
                return misalignment_output_res
            else:
                return
        except Exception as e:
            print(e)

    def mis_total_axis_check(self):
        misalignment_res = []
        try:

            result_x = self.mis_axis_check(
                self.avgfft_data_X, self.freqn_X, self.operating_rpm_X)
            result_y = self.mis_axis_check(
                self.avgfft_data_Y, self.freqn_Y, self.operating_rpm_X)
            result_z = self.mis_axis_check(
                self.avgfft_data_Z, self.freqn_Z, self.operating_rpm_X)

            if result_x == ['7'] or result_y == ['7']:
                mis_res = "7"
                mis_res_1 = "misalignment"
                misalignment_res.append(mis_res)
            else:
                mis_res = "8"
                mis_res_1 = " no misalignment"
                misalignment_res.append(mis_res)
            data = [self.Equipment_id, self.Sensor_id, "F", json.dumps(
                misalignment_res) + "$T$" + str(time.time())]
            
            # Stop individual data publish
            # v_helper.helper.data_to_publish.append(data)
            return misalignment_res

           
        except Exception as e:
            print(e)



# UNBALANCE
INI_WINDOWSIZE = 0
END_THRESHOLD = float(1.2)
SECONDS = int(60)
ANGLE_IN_DEGREE = int(360)
IDEAL_DEGREE = int(90)
DEGREE_RANGE = int(30)
START_FREQ_LIMIT = 0.8
END_FREQ_LIMIT  = 1.2
HARMONIC_ALLOWANCE = 0.2
MAX_AMP_LIMIT_1X = 0.4
START = 2
START_RANGE = 0.0
END = 1.0
HARMONIC_END = 11
HARMONICS_RANGE = 0.5
HARM_END_RNG = 9
# MISALIIGNMENT AND LOOSENESS
FREQUENCY_LIMIT = float(0.20)
RMS_2X_THRESHOLD = float(0.40)
RMS_3X_THRESHOLD = float(0.30)
# BEARING AND GEAR
rmsthreshold = 0.08
ONE = 1
TWO = 2
THREE = 3
four = 4
limit=1.5
FIFTEN=0.15
THIRTY_PERCENT=0.3
THIRTY_FIVE_PERCENT=0.35
FORTY_PERCENT=0.40
FIFTY_PERCENT=0.5
ONE_FIFTY_PERCENT=1.5
ONE_TWENTY_PERCENT=1.2
# RPM 
FREQ_IN_HERTZ = 60 
START_THRESHOLD = float(0.30)
# FFT AVERAGING
RMS_THRESHOLD_RANGE = 3
##data
